"""

"""
from __future__ import division

# Venn Diagram widget in orange-bioinformatics has been superseded by
# Venn Diagram in Orange (in Visualize category). However until there is
# a stable release of Orange (on which we can depend) we leave this widget
# in palace and only allow it's use if the Orange installation does not yet
# have the replacement.

try:
    from Orange.OrangeWidgets.Visualize import OWVennDiagram
except ImportError:
    pass
else:
    raise ImportError

import math
import unicodedata

from collections import namedtuple, defaultdict, OrderedDict, Counter
from itertools import count
from xml.sax.saxutils import escape

import numpy

from PyQt4.QtGui import (
    QComboBox, QGraphicsScene, QGraphicsView, QGraphicsWidget,
    QGraphicsPathItem, QGraphicsTextItem, QPainterPath, QPainter,
    QTransform, QColor, QBrush, QPen, QStyle, QPalette,
    QApplication
)

from PyQt4.QtCore import Qt, QPointF, QRectF, QLineF
from PyQt4.QtCore import pyqtSignal as Signal

import Orange

from Orange.OrangeWidgets.OWWidget import OWWidget, Multiple

from Orange.OrangeWidgets import OWGUI, OWItemModels, OWColorPalette

NAME = "Venn Diagram"

ICON = "icons/VennDiagram.svg"

INPUTS = [("Data", Orange.data.Table, "setData", Multiple)]

OUTPUTS = [("Data", Orange.data.Table)]


_InputData = namedtuple("_InputData", ["key", "name", "table"])
_ItemSet = namedtuple("_ItemSet", ["key", "name", "title", "items"])


class OWVennDiagram(OWWidget):
    settingsList = ["selection", "autocommit", "inputhints"]

    def __init__(self, parent=None, signalManager=None,
                 title="Venn Diagram"):
        super(OWVennDiagram, self).__init__(parent, signalManager, title,
                                            wantGraph=True)

        self.autocommit = False
        # Selected disjoint subset indices
        self.selection = []

        # Stored input set hints
        # {(index, inputname, attributes): (selectedattrname, itemsettitle)}
        # The 'selectedattrname' can be None
        self.inputhints = {}

        self.loadSettings()

        # Output changed flag
        self._changed = False
        # Diagram update is in progress
        self._updating = False
        # Input update is in progress
        self._inputUpdate = False

        # Input datasets
        self.data = OrderedDict()
        # Extracted input item sets
        self.itemsets = OrderedDict()

        # GUI
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.info = OWGUI.widgetLabel(box, "No data on input\n")

        self.inputsBox = OWGUI.widgetBox(self.controlArea, "Input data sets")

        for i in range(5):
            box = OWGUI.widgetBox(self.inputsBox, "Input %i" % i, flat=True)
            model = OWItemModels.VariableListModel(parent=self)
            cb = QComboBox()
            cb.setModel(model)
            cb.activated[int].connect(self._on_inputAttrActivated)
            box.setEnabled(False)
            # Store the combo in the box for later use.
            box.combo_box = cb
            box.layout().addWidget(cb)

        OWGUI.rubber(self.controlArea)

        box = OWGUI.widgetBox(self.controlArea, "Output")
        cb = OWGUI.checkBox(box, self, "autocommit", "Commit on any change")
        b = OWGUI.button(box, self, "Commit", callback=self.commit,
                         default=True)
        OWGUI.setStopper(self, b, cb, "_changed", callback=self.commit)

        # Main area view
        self.scene = QGraphicsScene()
        self.view = QGraphicsView(self.scene)
        self.view.setRenderHint(QPainter.Antialiasing)
        self.view.setBackgroundRole(QPalette.Window)
        self.view.setFrameStyle(QGraphicsView.StyledPanel)

        self.mainArea.layout().addWidget(self.view)
        self.vennwidget = VennDiagram()
        self.vennwidget.resize(400, 400)
        self.vennwidget.itemTextEdited.connect(self._on_itemTextEdited)
        self.scene.selectionChanged.connect(self._on_selectionChanged)

        self.scene.addItem(self.vennwidget)

        self.resize(self.controlArea.sizeHint().width() + 550,
                    max(self.controlArea.sizeHint().height(), 550))

        self._queue = []
        self.graphButton.clicked.connect(self.saveImage)

    def setData(self, data, key=None):
        self.error(0)
        if not self._inputUpdate:
            self._storeHints()
            self._inputUpdate = True

        if key in self.data:
            if data is None:
                # Remove the input
                self._remove(key)
            else:
                # Update existing item
                self._update(key, data)
        elif data is not None:
            # TODO: Allow setting more them 5 inputs and let the user
            # select the 5 to display.
            if len(self.data) == 5:
                self.error(0, "Can only take 5 inputs.")
                return
            # Add a new input
            self._add(key, data)

    def handleNewSignals(self):
        self._inputUpdate = False
        incremental = all(inc for _, inc in self._queue)

        if incremental:
            self._updateItemsets()
        else:
            self._createItemsets()
            self._restoreHints()
            self._updateItemsets()

        del self._queue[:]

        self._createDiagram()
        if self.data:
            self.info.setText(
                "{} data sets on input.\n".format(len(self.data)))
        else:
            self.info.setText("No data on input\n")

        OWWidget.handleNewSignals(self)

    def _invalidate(self, keys=None, incremental=True):
        """
        Invalidate input for a list of input keys.
        """
        if keys is None:
            keys = self.data.keys()

        self._queue.extend((key, incremental) for key in keys)

    def itemsetAttr(self, key):
        index = self.data.keys().index(key)
        _, combo = self._controlAtIndex(index)
        model = combo.model()
        attr_index = combo.currentIndex()
        if attr_index >= 0:
            return model[attr_index]
        else:
            return None

    def _controlAtIndex(self, index):
        group_box = self.inputsBox.layout().itemAt(index).widget()
        combo = group_box.combo_box
        return group_box, combo

    def _setAttributes(self, index, attrs):
        box, combo = self._controlAtIndex(index)
        model = combo.model()

        if attrs is None:
            model[:] = []
            box.setEnabled(False)
        else:
            if model[:] != attrs:
                model[:] = attrs

            box.setEnabled(True)

    def _add(self, key, table):
        name = table.name
        index = len(self.data)
        attrs = source_attributes(table.domain)
        if not attrs:
            self.warning(
                index, "Input {} has no suitable attributes.".format(index))

        self.data[key] = _InputData(key, name, table)

        self._setAttributes(index, attrs)

        self._invalidate([key], incremental=False)

        item = self.inputsBox.layout().itemAt(index)
        box = item.widget()
        box.setTitle("Input: {}".format(name))

    def _remove(self, key):
        index = self.data.keys().index(key)

        # Clear possible warnings.
        self.warning(index)

        self._setAttributes(index, None)

        del self.data[key]

        layout = self.inputsBox.layout()
        item = layout.takeAt(index)
        layout.addItem(item)
        inputs = self.data.values()

        for i in range(5):
            box, _ = self._controlAtIndex(i)
            if i < len(inputs):
                title = "Input: {}".format(inputs[i].name)
            else:
                title = "Input {}".format(i)
            box.setTitle(title)

        self._invalidate([key], incremental=False)

    def _update(self, key, table):
        name = table.name
        index = self.data.keys().index(key)
        attrs = source_attributes(table.domain)
        if not attrs:
            self.warning(
                index, "Input {} has no suitable attributes.".format(index))
        else:
            # Clear possible warnings.
            self.warning(index)
        self.data[key] = self.data[key]._replace(name=name, table=table)

        self._setAttributes(index, attrs)
        self._invalidate([key])

        item = self.inputsBox.layout().itemAt(index)
        box = item.widget()
        box.setTitle("Input: {}".format(name))

    def _updateItemsets(self):
        assert self.data.keys() == self.itemsets.keys()
        for key, input in self.data.items():
            attr = self.itemsetAttr(key)
            if attr is not None:
                items = [str(inst[attr]) for inst in input.table
                         if not inst[attr].is_special()]
            else:
                items = []

            item = self.itemsets[key]
            item = item._replace(items=items)
            if item.name != input.name:
                item = item._replace(name=input.name, title=input.name)
            self.itemsets[key] = item

    def _createItemsets(self):
        olditemsets = dict(self.itemsets)
        self.itemsets.clear()
        for key, input in self.data.items():
            attr = self.itemsetAttr(key)
            if attr is not None:
                items = [str(inst[attr]) for inst in input.table
                         if not inst[attr].is_special()]
            else:
                items = []

            title = input.name
            if key in olditemsets and olditemsets[key].name == input.name:
                # Reuse the title (which might have been changed by the user)
                title = olditemsets[key].title

            itemset = _ItemSet(key=key, name=input.name, title=title,
                               items=items)
            self.itemsets[key] = itemset

    def _storeHints(self):
        if self.data:
            self.inputhints.clear()
            for i, (key, input) in enumerate(self.data.items()):
                attrs = source_attributes(input.table.domain)
                attrs = tuple(attr.name for attr in attrs)
                selected = self.itemsetAttr(key)
                if selected is not None:
                    attr_name = selected.name
                else:
                    attr_name = None
                itemset = self.itemsets[key]
                self.inputhints[(i, input.name, attrs)] = \
                    (attr_name, itemset.title)

    def _restoreHints(self):
        settings = []
        for i, (key, input) in enumerate(self.data.items()):
            attrs = source_attributes(input.table.domain)
            attrs = tuple(attr.name for attr in attrs)
            hint = self.inputhints.get((i, input.name, attrs), None)
            if hint is not None:
                attr, name = hint
                attr_ind = attrs.index(attr) if attr is not None else -1
                settings.append((attr_ind, name))
            else:
                return

        # all inputs match the stored hints
        for i, key in enumerate(self.itemsets):
            attr, itemtitle = settings[i]
            self.itemsets[key] = self.itemsets[key]._replace(title=itemtitle)
            _, cb = self._controlAtIndex(i)
            cb.setCurrentIndex(attr)

    def _createDiagram(self):
        self._updating = True

        oldselection = list(self.selection)

        self.vennwidget.clear()
        n = len(self.itemsets)
        self.disjoint = disjoint(set(s.items) for s in self.itemsets.values())

        vennitems = []
        colors = OWColorPalette.ColorPaletteHSV(n)

        for i, (key, item) in enumerate(self.itemsets.items()):
            gr = VennSetItem(text=item.title, count=len(item.items))
            color = colors[i]
            color.setAlpha(100)
            gr.setBrush(QBrush(color))
            gr.setPen(QPen(Qt.NoPen))
            vennitems.append(gr)

        self.vennwidget.setItems(vennitems)

        for i, area in enumerate(self.vennwidget.vennareas()):
            area_items = list(self.disjoint[i])
            if i:
                area.setText("{0}".format(len(area_items)))

            label = disjoint_set_label(i, n, simplify=False)
            head = u"<h4>|{}| = {}</h4>".format(label, len(area_items))
            if len(area_items) > 32:
                items_str = u", ".join(map(escape, area_items[:32]))
                hidden = len(area_items) - 32
                tooltip = (u"{}<span>{}, ...</br>({} items not shown)<span>"
                           .format(head, items_str, hidden))
            elif area_items:
                tooltip = u"{}<span>{}</span>".format(
                    head,
                    u", ".join(map(escape, area_items))
                )
            else:
                tooltip = head

            area.setToolTip(tooltip)

            area.setPen(QPen(QColor(10, 10, 10, 200), 1.5))
            area.setFlag(QGraphicsPathItem.ItemIsSelectable, True)
            area.setSelected(i in oldselection)

        self._updating = False
        self._on_selectionChanged()

    def _on_selectionChanged(self):
        if self._updating:
            return

        areas = self.vennwidget.vennareas()
        indices = [i for i, area in enumerate(areas)
                   if area.isSelected()]

        self.selection = indices

        self.invalidateOutput()

    def _on_inputAttrActivated(self, attr_index):
        combo = self.sender()
        # Find the input index to which the combo box belongs
        # (they are reordered when removing inputs).
        index = None
        inputs = self.data.items()
        for i in range(len(inputs)):
            _, c = self._controlAtIndex(i)
            if c is combo:
                index = i
                break

        assert (index is not None)

        key, _ = inputs[index]

        self._invalidate([key])
        self._updateItemsets()
        self._createDiagram()

    def _on_itemTextEdited(self, index, text):
        text = str(text)
        key = self.itemsets.keys()[index]
        self.itemsets[key] = self.itemsets[key]._replace(title=text)

    def invalidateOutput(self):
        if self.autocommit:
            self.commit()
        else:
            self._changed = True

    def commit(self):
        selected_subsets = []

        selected_items = reduce(
            set.union, [self.disjoint[index] for index in self.selection],
            set()
        )

        def match(val):
            if val.is_special():
                return False
            else:
                return str(val) in selected_items

        source_mid = Orange.core.newmetaid()
        source_var = Orange.feature.String("source")
        item_mid = Orange.core.newmetaid()
        item_id_var = Orange.feature.String("item_id")

        names = [itemset.title.strip() for itemset in self.itemsets.values()]
        names = uniquify(names)

        for i, (key, input) in enumerate(self.data.items()):
            attr = self.itemsetAttr(key)
            if attr is not None:
                mask = map(match, (inst[attr] for inst in input.table))
            else:
                mask = [False] * len(input.table)

            subset = Orange.data.Table(input.table.domain.clone(),
                                       input.table.select(mask))

            assert subset.domain is not input.table.domain

            # add a column with source table id to the data
            subset.domain.add_meta(source_mid, source_var)
            subset.add_meta_attribute(source_var, str(names[i]))
            # add a column with instance set id
            subset.domain.add_meta(item_mid, item_id_var)
            subset.add_meta_attribute(item_mid)
            for inst in subset:
                inst[item_id_var] = str(inst[attr])

            if subset:
                selected_subsets.append(subset)

        if selected_subsets:
            data = table_concat(selected_subsets)
            # Get all variables which are not constant between the same
            # item set
            varying = varying_between(data, [item_id_var])
            if source_var in varying:
                varying.remove(source_var)

            data = reshape_wide(data, varying, [item_id_var], [source_var])
            # remove the temporary item set id column
            data.remove_meta_attribute(item_id_var)
            data.domain.remove_meta(item_id_var)
        else:
            data = None

        self.send("Data", data)

    def saveImage(self):
        from Orange.OrangeWidgets.OWDlgs import OWChooseImageSizeDlg
        dlg = OWChooseImageSizeDlg(self.scene, parent=self)
        dlg.exec_()

    def getSettings(self, *args, **kwargs):
        self._storeHints()
        return OWWidget.getSettings(self, *args, **kwargs)


def table_concat(tables):
    """
    Concatenate a list of tables.

    The resulting table will have a union of all attributes of `tables`.

    """
    features = []
    features_seen = set()
    class_var = None
    metas = {}
    metas_seen = set()

    for table in tables:
        features.extend(f for f in table.domain.features
                        if f not in features_seen)
        features_seen.update(features)

        new_metas = {}

        if table.domain.class_var is not None:
            if class_var is not None and table.domain.class_var != class_var:
                # Move the extra class variables to meta attributes
                if table.domain.class_var not in metas_seen:
                    new_metas[Orange.core.newmetaid()] = table.domain.class_var
                    metas_seen.add(table.domain.class_var)
            else:
                class_var = table.domain.class_var
                features_seen.add(class_var)

        new_metas.update(
            {mid: meta for mid, meta in table.domain.getmetas().items()
             if meta not in metas_seen}
        )

        metas_seen.update(new_metas.itervalues())

        metas.update(new_metas)

    domain = Orange.data.Domain(features, class_var)
    domain.addmetas(metas)

    new_table = Orange.data.Table(domain)
    for table in tables:
        new_table.extend(table.translate(domain))

    return new_table


def copy_descriptor(descriptor, newname=None):
    """
    Create a copy of the descriptor.

    If newname is not None the new descriptor will have the same
    name as the input.

    """
    if newname is None:
        newname = descriptor.name

    if isinstance(descriptor, Orange.feature.Discrete):
        newf = Orange.feature.Discrete(
            newname,
            values=descriptor.values,
            base_value=descriptor.base_value,
            ordered=descriptor.ordered,
            attributes=dict(descriptor.attributes))

    elif isinstance(descriptor, Orange.feature.Continuous):
        newf = Orange.feature.Continuous(
            newname,
            number_of_decimals=descriptor.number_of_decimals,
            scientific_format=descriptor.scientific_format,
            adjust_decimals=descriptor.adjust_decimals,
            attributes=dict(descriptor.attributes))

    else:
        newf = type(descriptor)(
            newname,
            attributes=dict(descriptor.attributes))

    return newf


def reshape_wide(table, varlist, idvarlist, groupvarlist):
    """
    Reshape a data table into a wide format.

    :param Orange.data.Table table:
        Source data table in long format.
    :param varlist:
        A list of variables to reshape.
    :param list idvarlist:
        A list of variables in `table` uniquely identifying multiple
        records in `table` (i.e. subject id).
    :param groupvarlist:
        A list of variables differentiating multiple records
        (i.e. conditions).

    """
    def inst_key(inst, vars):
        return tuple(str(inst[var]) for var in vars)

    instance_groups = [inst_key(inst, groupvarlist) for inst in table]
    # A list of groups (for each element in a group the varying variable
    # will be duplicated)
    groups = list(unique(instance_groups))
    group_names = [", ".join(group) for group in groups]

    # A list of instance ids (subject ids)
    # Each instance in the output will correspond to one of these ids)
    instance_ids = [inst_key(inst, idvarlist) for inst in table]
    ids = list(unique(instance_ids))

    # an mapping from ids to an list of input instance indices
    # each instance in this list belongs to one group (but not all
    # groups need to be present).
    inst_by_id = defaultdict(list)

    for i, inst in enumerate(table):
        inst_id = instance_ids[i]
        inst_by_id[inst_id].append(i)

    newfeatures = []
    newmetas = []
    expanded_features = {}

    def expanded(feat):
        return [copy_descriptor(feat, newname="%s (%s)" %
                                (feat.name, group_name))
                for group_name in group_names]

    for feat in table.domain.variables:
        if feat in varlist:
            features = expanded(feat)
            newfeatures.extend(features)
            expanded_features[feat] = dict(zip(groups, features))
        elif feat not in groupvarlist:
            newfeatures.append(feat)

    for mid, meta in table.domain.getmetas().items():
        if meta in varlist:
            metas = expanded(meta)
            newmetas.extend((Orange.core.newmetaid(), meta)
                            for meta in metas)
            expanded_features[meta] = dict(zip(groups, metas))
        elif meta not in groupvarlist:
            newmetas.append((mid, meta))

    if table.domain.class_var and table.domain.class_var is newfeatures[-1]:
        new_class_var = table.domain.class_var
        newfeatures = newfeatures[:-1]
    else:
        new_class_var = None

    domain = Orange.data.Domain(newfeatures, new_class_var)
    domain.addmetas(dict(newmetas))

    newtable = Orange.data.Table(domain)

    for inst_id in ids:
        indices = inst_by_id[inst_id]
        inst = Orange.data.Instance(domain, table[indices[0]])
        for index in indices:
            source_inst = table[index]
            group = instance_groups[index]
            for source_var in varlist:
                newf = expanded_features[source_var][group]
                inst[newf] = source_inst[source_var].value
        newtable.append(inst)

    return newtable


def unique(seq):
    """
    Return an iterator over unique items of `seq`.

    .. note:: Items must be hashable.

    """
    seen = set()
    for item in seq:
        if item not in seen:
            yield item
            seen.add(item)


def varying_between(table, idvarlist):
    """
    Return a list of all variables with non constant values between
    groups defined by `idvarlist`.

    """
    def inst_key(inst, vars):
        return tuple(str(inst[var]) for var in vars)

    excluded = set(idvarlist)
    all_possible = [var for var in (table.domain.variables +
                                    table.domain.getmetas().values())
                    if var not in excluded]
    candidate_set = set(all_possible)

    idmap = Orange.data.utils.table_map(table, idvarlist)
    values = {}
    varying = set()
    for indices in idmap.values():
        for var in list(candidate_set):
            values = [table[i][var].native() for i in indices]
            if len(set(values)) != 1:
                varying.add(var)
                candidate_set.remove(var)

    return sorted(varying, key=all_possible.index)


def uniquify(strings):
    """
    Return a list of unique strings.

    The string at i'th position will have the same prefix as strings[i]
    with an appended suffix to make the item unique (if necessary).

    >>> uniquify(["cat", "dog", "cat"])
    ["cat 1", "dog", "cat 2"]

    """
    counter = Counter(strings)
    counts = defaultdict(count)
    newstrings = []
    for string in strings:
        if counter[string] > 1:
            newstrings.append(string + (" %i" % (next(counts[string]) + 1)))
        else:
            newstrings.append(string)

    return newstrings


def string_attributes(domain):
    """
    Return all string attributes from the domain.
    """
    return [attr for attr in domain.variables +
                domain.getmetas().values()
            if isinstance(attr, Orange.feature.String)]


def discrete_attributes(domain):
    """
    Return all discrete attributes from the domain.
    """
    return filter(lambda attr: isinstance(attr, Orange.feature.Discrete),
                  domain.variables + domain.getmetas().values())


def source_attributes(domain):
    """
    Return all suitable attributes for the venn diagram.
    """
    return string_attributes(domain) + discrete_attributes(domain)


def disjoint(sets):
    """
    Return all disjoint subsets.
    """
    sets = list(sets)
    n = len(sets)
    disjoint_sets = [None] * (2 ** n)
    for i in range(2 ** n):
        key = setkey(i, n)
        included = [s for s, inc in zip(sets, key) if inc]
        excluded = [s for s, inc in zip(sets, key) if not inc]
        if any(included):
            s = reduce(set.intersection, included)
        else:
            s = set()

        s = reduce(set.difference, excluded, s)

        disjoint_sets[i] = s

    return disjoint_sets


def disjoint_set_label(i, n, simplify=False):
    """
    Return a html formated label for a disjoint set indexed by `i`.
    """
    intersection = unicodedata.lookup("INTERSECTION")
    # comp = unicodedata.lookup("COMPLEMENT")  #
    # This depends on the font but the unicode complement in
    # general does not look nice in a super script so we use
    # plain c instead.
    comp = "c"

    def label_for_index(i):
        return chr(ord("A") + i)

    if simplify:
        return "".join(label_for_index(i) for i, b in enumerate(setkey(i, n))
                       if b)
    else:
        return intersection.join(label_for_index(i) +
                                 ("" if b else "<sup>" + comp + "</sup>")
                                 for i, b in enumerate(setkey(i, n)))


class VennSetItem(QGraphicsPathItem):
    def __init__(self, parent=None, text=None, count=None):
        super(VennSetItem, self).__init__(parent)
        self.text = text
        self.count = count


# TODO: Use palette's selected/highligted text / background colors to
# indicate selection

class VennIntersectionArea(QGraphicsPathItem):
    def __init__(self, parent=None, text=""):
        super(QGraphicsPathItem, self).__init__(parent)
        self.setAcceptHoverEvents(True)
        self.setPen(QPen(Qt.NoPen))

        self.text = QGraphicsTextItem(self)
        layout = self.text.document().documentLayout()
        layout.documentSizeChanged.connect(self._onLayoutChanged)

        self._text = ""
        self._anchor = QPointF()

    def setText(self, text):
        if self._text != text:
            self._text = text
            self.text.setPlainText(text)

    def text(self):
        return self._text

    def setTextAnchor(self, pos):
        if self._anchor != pos:
            self._anchor = pos
            self._updateTextAnchor()

    def hoverEnterEvent(self, event):
        self.setZValue(self.zValue() + 1)
        return QGraphicsPathItem.hoverEnterEvent(self, event)

    def hoverLeaveEvent(self, event):
        self.setZValue(self.zValue() - 1)
        return QGraphicsPathItem.hoverLeaveEvent(self, event)

#     def mousePressEvent(self, event):
#         pos = event.pos()
#         parent = self.parentItem()
#         pbrect = parent.boundingRect()
#         w, h = pbrect.width(), pbrect.height()
#         print "(%.3f, %.3f)" % (pos.x() / w, pos.y() / h)
#         super(VennIntersectionArea, self).mousePressEvent(event)

    def paint(self, painter, option, widget=None):
        painter.save()
        path = self.path()
        brush = QBrush(self.brush())
        pen = QPen(self.pen())

        if option.state & QStyle.State_Selected:
            pen.setColor(Qt.red)
            brush.setStyle(Qt.DiagCrossPattern)
            brush.setColor(QColor(40, 40, 40, 100))

        elif option.state & QStyle.State_MouseOver:
            pen.setColor(Qt.blue)

        if option.state & QStyle.State_MouseOver:
            brush.setColor(QColor(100, 100, 100, 100))
            if brush.style() == Qt.NoBrush:
                # Make sure the highlight is actually visible.
                brush.setStyle(Qt.SolidPattern)

        painter.setPen(pen)
        painter.setBrush(brush)
        painter.drawPath(path)
        painter.restore()

    def itemChange(self, change, value):
        if change == QGraphicsPathItem.ItemSelectedHasChanged:
            if value.toBool():
                self.setZValue(self.zValue() + 1)
            else:
                self.setZValue(self.zValue() - 1)

        return QGraphicsPathItem.itemChange(self, change, value)

    def _updateTextAnchor(self):
        rect = self.text.boundingRect()
        pos = anchor_rect(rect, self._anchor)
        self.text.setPos(pos)

    def _onLayoutChanged(self):
        self._updateTextAnchor()


class GraphicsTextEdit(QGraphicsTextItem):
    #: Edit triggers
    NoEditTriggers, DoubleClicked = 0, 1

    editingFinished = Signal()
    editingStarted = Signal()

    documentSizeChanged = Signal()

    def __init__(self, *args, **kwargs):
        super(GraphicsTextEdit, self).__init__(*args, **kwargs)
        self.setTabChangesFocus(True)
        self._edittrigger = GraphicsTextEdit.DoubleClicked
        self._editing = False
        self.document().documentLayout().documentSizeChanged.connect(
            self.documentSizeChanged
        )

    def mouseDoubleClickEvent(self, event):
        super(GraphicsTextEdit, self).mouseDoubleClickEvent(event)
        if self._edittrigger == GraphicsTextEdit.DoubleClicked:
            self._start()

    def focusOutEvent(self, event):
        super(GraphicsTextEdit, self).focusOutEvent(event)

        if self._editing:
            self._end()

    def _start(self):
        self._editing = True
        self.setTextInteractionFlags(Qt.TextEditorInteraction)
        self.setFocus(Qt.MouseFocusReason)
        self.editingStarted.emit()

    def _end(self):
        self._editing = False
        self.setTextInteractionFlags(Qt.NoTextInteraction)
        self.editingFinished.emit()


class VennDiagram(QGraphicsWidget):
    # rect and petal are for future work
    Circle, Ellipse, Rect, Petal = 1, 2, 3, 4

    TitleFormat = "<center><h4>{0}</h4>{1}</center>"

    selectionChanged = Signal()
    itemTextEdited = Signal(int, str)

    def __init__(self, parent=None):
        super(VennDiagram, self).__init__(parent)
        self.shapeType = VennDiagram.Circle

        self._setup()

    def _setup(self):
        self._items = []
        self._vennareas = []
        self._textitems = []

    def item(self, index):
        return self._items[index]

    def items(self):
        return list(self._items)

    def count(self):
        return len(self._items)

    def setItems(self, items):
        if self._items:
            self.clear()

        self._items = list(items)

        for item in self._items:
            item.setParentItem(self)
            item.setVisible(True)

        fmt = self.TitleFormat.format

        font = self.font()
        font.setPixelSize(14)

        for item in items:
            text = GraphicsTextEdit(self)
            text.setFont(font)
            text.setDefaultTextColor(QColor("#333"))
            text.setHtml(fmt(escape(item.text), item.count))
            text.adjustSize()
            text.editingStarted.connect(self._on_editingStarted)
            text.editingFinished.connect(self._on_editingFinished)
            text.documentSizeChanged.connect(
                self._on_itemTextSizeChanged
            )

            self._textitems.append(text)

        self._vennareas = [
            VennIntersectionArea(parent=self)
            for i in range(2 ** len(items))
        ]
        self._subsettextitems = [
            QGraphicsTextItem(parent=self)
            for i in range(2 ** len(items))
        ]

        self._updateLayout()

    def clear(self):
        scene = self.scene()
        items = self.vennareas() + self.items() + self._textitems

        for item in self._textitems:
            item.editingStarted.disconnect(self._on_editingStarted)
            item.editingFinished.disconnect(self._on_editingFinished)
            item.documentSizeChanged.disconnect(
                self._on_itemTextSizeChanged
            )

        self._items = []
        self._vennareas = []
        self._textitems = []

        for item in items:
            item.setVisible(False)
            item.setParentItem(None)
            if scene is not None:
                scene.removeItem(item)

    def vennareas(self):
        return list(self._vennareas)

    def setFont(self, font):
        if self._font != font:
            self.prepareGeometryChange()
            self._font = font

            for item in self.items():
                item.setFont(font)

    def _updateLayout(self):
        rect = self.geometry()
        n = len(self._items)
        if not n:
            return

        regions = venn_diagram(n, shape=self.shapeType)

        # The y axis in Qt points downward
        transform = QTransform().scale(1, -1)
        regions = map(transform.map, regions)

        union_brect = reduce(QRectF.united,
                             (path.boundingRect() for path in regions))

        scalex = rect.width() / union_brect.width()
        scaley = rect.height() / union_brect.height()
        scale = min(scalex, scaley)

        transform = QTransform().scale(scale, scale)

        regions = [transform.map(path) for path in regions]

        center = rect.width() / 2, rect.height() / 2
        for item, path in zip(self.items(), regions):
            item.setPath(path)
            item.setPos(*center)

        intersections = venn_intersections(regions)
        assert len(intersections) == 2 ** n
        assert len(self.vennareas()) == 2 ** n

        anchors = [(0, 0)] + subset_anchors(self._items)

        anchor_transform = QTransform().scale(rect.width(), -rect.height())
        for i, area in enumerate(self.vennareas()):
            area.setPath(intersections[setkey(i, n)])
            area.setPos(*center)
            x, y = anchors[i]
            anchor = anchor_transform.map(QPointF(x, y))
            area.setTextAnchor(anchor)
            area.setZValue(30)

        self._updateTextAnchors()

    def _updateTextAnchors(self):
        n = len(self._items)

        items = self._items
        dist = 15

        shape = reduce(QPainterPath.united, [item.path() for item in items])
        brect = shape.boundingRect()
        bradius = max(brect.width() / 2, brect.height() / 2)

        center = self.boundingRect().center()

        anchors = _category_anchors(items)
        self._textanchors = []
        for angle, anchor_h, anchor_v in anchors:
            line = QLineF.fromPolar(bradius, angle)
            ext = QLineF.fromPolar(dist, angle)
            line = QLineF(line.p1(), line.p2() + ext.p2())
            line = line.translated(center)

            anchor_pos = line.p2()
            self._textanchors.append((anchor_pos, anchor_h, anchor_v))

        for i in range(n):
            self._updateTextItemPos(i)

    def _updateTextItemPos(self, i):
        item = self._textitems[i]
        anchor_pos, anchor_h, anchor_v = self._textanchors[i]
        rect = item.boundingRect()
        pos = anchor_rect(rect, anchor_pos, anchor_h, anchor_v)
        item.setPos(pos)

    def setGeometry(self, geometry):
        super(VennDiagram, self).setGeometry(geometry)
        self._updateLayout()

    def paint(self, painter, option, w):
        super(VennDiagram, self).paint(painter, option, w)
#         painter.drawRect(self.boundingRect())

    def _on_editingStarted(self):
        item = self.sender()
        index = self._textitems.index(item)
        text = self._items[index].text
        item.setTextWidth(-1)
        item.setHtml(self.TitleFormat.format(escape(text), "<br/>"))

    def _on_editingFinished(self):
        item = self.sender()
        index = self._textitems.index(item)
        text = item.toPlainText()
        if text != self._items[index].text:
            self._items[index].text = text

            self.itemTextEdited.emit(index, text)

        item.setHtml(
            self.TitleFormat.format(escape(text), self._items[index].count))
        item.adjustSize()

    def _on_itemTextSizeChanged(self):
        item = self.sender()
        index = self._textitems.index(item)
        self._updateTextItemPos(index)


def anchor_rect(rect, anchor_pos,
                anchor_h=Qt.AnchorHorizontalCenter,
                anchor_v=Qt.AnchorVerticalCenter):

    if anchor_h == Qt.AnchorLeft:
        x = anchor_pos.x()
    elif anchor_h == Qt.AnchorHorizontalCenter:
        x = anchor_pos.x() - rect.width() / 2
    elif anchor_h == Qt.AnchorRight:
        x = anchor_pos.x() - rect.width()
    else:
        raise ValueError(anchor_h)

    if anchor_v == Qt.AnchorTop:
        y = anchor_pos.y()
    elif anchor_v == Qt.AnchorVerticalCenter:
        y = anchor_pos.y() - rect.height() / 2
    elif anchor_v == Qt.AnchorBottom:
        y = anchor_pos.y() - rect.height()
    else:
        raise ValueError(anchor_v)

    return QPointF(x, y)


def radians(angle):
    return 2 * math.pi * angle / 360


def unit_point(x, r=1.0):
    x = radians(x)
    return (r * math.cos(x), r * math.sin(x))


def _category_anchors(shapes):
    n = len(shapes)
    return _CATEGORY_ANCHORS[n - 1]


# (angle, horizontal anchor, vertical anchor)
_CATEGORY_ANCHORS = (
    # n == 1
    ((90, Qt.AnchorHorizontalCenter, Qt.AnchorBottom),),
    # n == 2
    ((180, Qt.AnchorRight, Qt.AnchorVerticalCenter),
     (0, Qt.AnchorLeft, Qt.AnchorVerticalCenter)),
    # n == 3
    ((150, Qt.AnchorRight, Qt.AnchorBottom),
     (30, Qt.AnchorLeft, Qt.AnchorBottom),
     (270, Qt.AnchorHorizontalCenter, Qt.AnchorTop)),
    # n == 4
    ((270 + 45, Qt.AnchorLeft, Qt.AnchorTop),
     (270 - 45, Qt.AnchorRight, Qt.AnchorTop),
     (90 - 15, Qt.AnchorLeft, Qt.AnchorBottom),
     (90 + 15, Qt.AnchorRight, Qt.AnchorBottom)),
    # n == 5
    ((90 - 5, Qt.AnchorHorizontalCenter, Qt.AnchorBottom),
     (18 - 5, Qt.AnchorLeft, Qt.AnchorVerticalCenter),
     (306 - 5, Qt.AnchorLeft, Qt.AnchorTop),
     (234 - 5, Qt.AnchorRight, Qt.AnchorTop),
     (162 - 5, Qt.AnchorRight, Qt.AnchorVerticalCenter),)
)


def subset_anchors(shapes):
    n = len(shapes)
    if n == 1:
        return [(0, 0)]
    elif n == 2:
        return [unit_point(180, r=1/3),
                unit_point(0, r=1/3),
                (0, 0)]
    elif n == 3:
        return [unit_point(150, r=0.35),  # A
                unit_point(30, r=0.35),   # B
                unit_point(90, r=0.27),   # AB
                unit_point(270, r=0.35),  # C
                unit_point(210, r=0.27),  # AC
                unit_point(330, r=0.27),  # BC
                unit_point(0, r=0),       # ABC
                ]
    elif n == 4:
        anchors = [
            (0.400, 0.110),    # A
            (-0.400, 0.110),   # B
            (0.000, -0.285),   # AB
            (0.180, 0.330),    # C
            (0.265, 0.205),    # AC
            (-0.240, -0.110),  # BC
            (-0.100, -0.190),  # ABC
            (-0.180, 0.330),   # D
            (0.240, -0.110),   # AD
            (-0.265, 0.205),   # BD
            (0.100, -0.190),   # ABD
            (0.000, 0.250),    # CD
            (0.153, 0.090),    # ACD
            (-0.153, 0.090),   # BCD
            (0.000, -0.060),   # ABCD
        ]
        return anchors

    elif n == 5:
        anchors = [None] * 32
        # Base anchors
        A = (0.033, 0.385)
        AD = (0.095, 0.250)
        AE = (-0.100, 0.265)
        ACE = (-0.130, 0.220)
        ADE = (0.010, 0.225)
        ACDE = (-0.095, 0.175)
        ABCDE = (0.0, 0.0)

        anchors[-1] = ABCDE

        bases = [(0b00001, A),
                 (0b01001, AD),
                 (0b10001, AE),
                 (0b10101, ACE),
                 (0b11001, ADE),
                 (0b11101, ACDE)]

        for i in range(5):
            for index, anchor in bases:
                index = bit_rot_left(index, i, bits=5)
                assert anchors[index] is None
                anchors[index] = rotate_point(anchor, - 72 * i)

        assert all(anchors[1:])
        return anchors[1:]


def bit_rot_left(x, y, bits=32):
    mask = 2 ** bits - 1
    x_masked = x & mask
    return (x << y) & mask | (x_masked >> bits - y)


def rotate_point(p, angle):
    r = radians(angle)
    R = numpy.array([[math.cos(r), -math.sin(r)],
                     [math.sin(r), math.cos(r)]])
    x, y = numpy.dot(R, p)
    return (float(x), float(y))


def line_extended(line, distance):
    """
    Return an QLineF extended by `distance` units in the positive direction.
    """
    angle = line.angle() / 360 * 2 * math.pi
    dx, dy = unit_point(angle, r=distance)
    return QLineF(line.p1(), line.p2() + QPointF(dx, dy))


def circle_path(center, r=1.0):
    return ellipse_path(center, r, r, rotation=0)


def ellipse_path(center, a, b, rotation=0):
    if not isinstance(center, QPointF):
        center = QPointF(*center)

    brect = QRectF(-a, -b, 2 * a, 2 * b)

    path = QPainterPath()
    path.addEllipse(brect)

    if rotation != 0:
        transform = QTransform().rotate(rotation)
        path = transform.map(path)

    path.translate(center)
    return path


# TODO: Should include anchors for text layout (both inside and outside).
# for each item {path: QPainterPath,
#                text_anchors: [{center}] * (2 ** n)
#                mayor_axis: QLineF,
#                boundingRect QPolygonF (with 4 vertices)}
#
# Should be a new class with overloads for ellipse/circle, rect, and petal
# shapes, should store all constructor parameters, rotation, center,
# mayor/minor axis.


def venn_diagram(n, shape=VennDiagram.Circle):
    if n < 1 or n > 5:
        raise ValueError()

    paths = []

    if n == 1:
        paths = [circle_path(center=(0, 0), r=0.5)]
    elif n == 2:
        angles = [180, 0]
        paths = [circle_path(center=unit_point(x, r=1/6), r=1/3)
                 for x in angles]
    elif n == 3:
        angles = [150 - 120 * i for i in range(3)]
        paths = [circle_path(center=unit_point(x, r=1/6), r=1/3)
                 for x in angles]
    elif n == 4:
        # Constants shamelessly stolen from VennDiagram R package
        paths = [
            ellipse_path((0.65 - 0.5, 0.47 - 0.5), 0.35, 0.20, 45),
            ellipse_path((0.35 - 0.5, 0.47 - 0.5), 0.35, 0.20, 135),
            ellipse_path((0.5 - 0.5, 0.57 - 0.5), 0.35, 0.20, 45),
            ellipse_path((0.5 - 0.5, 0.57 - 0.5), 0.35, 0.20, 134),
        ]
    elif n == 5:
        # Constants shamelessly stolen from VennDiagram R package
        d = 0.13
        a, b = 0.24, 0.48
        a, b = b, a
        a, b = 0.48, 0.24
        paths = [ellipse_path(unit_point((1 - i) * 72, r=d),
                              a, b, rotation=90 - (i * 72))
                 for i in range(5)]

    return paths


def setkey(intval, n):
    return tuple(bool(intval & (2 ** i)) for i in range(n))


def keyrange(n):
    if n < 0:
        raise ValueError()

    for i in range(2 ** n):
        yield setkey(i, n)


def venn_intersections(paths):
    n = len(paths)
    return {key: venn_intersection(paths, key) for key in keyrange(n)}


def venn_intersection(paths, key):
    if not any(key):
        return QPainterPath()

    # first take the intersection of all included paths
    path = reduce(QPainterPath.intersected,
                  (path for path, included in zip(paths, key) if included))

    # subtract all the excluded sets (i.e. take the intersection
    # with the excluded set complements)
    path = reduce(QPainterPath.subtracted,
                  (path for path, included in zip(paths, key) if not included),
                  path)

    return path


def test():
    app = QApplication([])
    w = OWVennDiagram()
    data = Orange.data.Table("brown-selected")

    data.domain.addmetas({-42: Orange.feature.String("Test")})
    for i, inst in enumerate(data):
        inst[-42] = "{}".format(i % 30)

    random = Orange.misc.Random()
    indices = Orange.data.sample.SubsetIndices2(
        p0=0.7, random_generator=random)

    d1 = data.select(indices(data))
    d2 = data.select(indices(data))
    d3 = data.select(indices(data))
    d4 = data.select(indices(data))
    d5 = data.select(indices(data))

    n = 5
    for i, data in zip(range(n), [d1, d2, d3, d4, d5]):
        data.name = chr(ord("A") + i)
        w.setData(data, key=i)

    w.handleNewSignals()
    w.show()
    app.exec_()

    del w
    app.processEvents()
    return app


def test1():
    app = QApplication([])
    w = OWVennDiagram()
    data1 = Orange.data.Table("brown-selected")
    data2 = Orange.data.Table("brown-selected")
    w.setData(data1, 1)
    w.setData(data2, 2)
    w.handleNewSignals()

    w.show()
    app.exec_()

    del w
    return app

if __name__ == "__main__":
    test()



import os
import urllib2
import shutil
import pickle

from collections import defaultdict

from Orange.orng import orngServerFiles

from Orange.utils.serverfiles import localpath_download

domain = "dictybase"
pickle_file = "mutants.pkl"
tags = ["Dictyostelium discoideum", "mutant", "dictyBase", "phenotype"]


class DictyMutant(object):
    """
    A single Dictyostelium discoideum mutant from the Dictybase.
   
    :param str mutant_entry: A single mutant entry from 
        `curated mutants file <http://dictybase.org/db/cgi-bin/dictyBase/download/download.pl?area=mutant_phenotypes&ID=all-mutants.txt>`_.

    :ivar DictyMutant.name: dictyBase ID.
    :ivar DictyMutant.descriptor: dictyBase strain descriptor.
    :ivar DictyMutant.genes: all associated genes.
    :ivar DictyMutant.phenotypes: all associated phenotypes.

    """
    def __init__(self, mutant_entry):
        mutant = mutant_entry.split("\t")
        self.name = mutant[0]
        self.descriptor = mutant[1]
        self.genes = mutant[2].split(" | ")
        self.phenotypes = mutant[3].split(" | ")
        self.null = False
        self.overexp = False
        self.multiple = False
        self.develop = False
        self.other = False


class DictyMutants(object):
    """
    A collection of Dictybase mutants as a dictionary of
    :obj:`DictyMutant` objects.

    :param local_database_path: A path for storing
        D. dictyostelium mutants objects. If `None` then 
        a default database path is used.
    
    """
    
    VERSION=1
    DEFAULT_DATABASE_PATH = orngServerFiles.localpath("DictyMutants") #use a default local folder for storing the genesets
    
    def __init__(self, local_database_path=None):
        self.local_database_path = local_database_path if local_database_path is not None else self.DEFAULT_DATABASE_PATH
        
        if not os.path.exists(self.local_database_path):
            os.mkdir(self.local_database_path)
            
        self._mutants = pickle.load(open(localpath_download(domain, pickle_file), "rb"))
              
    def update_file(self, name):
        url = "http://dictybase.org/db/cgi-bin/dictyBase/download/download.pl?area=mutant_phenotypes&ID="
        filename = os.path.join(self.local_database_path, name)
        temp_file = os.path.join(self.local_database_path, name + "_temp")
        stream = urllib2.urlopen(url + name)
    
        with open(temp_file, "wb") as file:
            shutil.copyfileobj(stream, file)
    
        os.rename(temp_file, filename)
        return filename
    
    def load_mutants(self, file):
        data = open(file)
        #data_header = data.readline()
        data = data.read()
        return data.splitlines()
                 
    def download_mutants(self):   
        all_mutants = self.load_mutants(self.update_file("all-mutants.txt"))
        null_mutants = self.load_mutants(
            self.update_file("null-mutants.txt"))
        overexp_mutants = self.load_mutants(
            self.update_file("overexpression-mutants.txt"))
        multiple_mutants = self.load_mutants(
            self.update_file("multiple-mutants.txt"))
        develop_mutants = self.load_mutants(
            self.update_file("developmental-mutants.txt"))
        other_mutants = self.load_mutants(
            self.update_file("other-mutants.txt"))
   
        _mutants = [DictyMutant(mutant) for mutant in all_mutants]
        
        the_nulls = set([DictyMutant(line).name for line in null_mutants])
        the_overexps = set([DictyMutant(line).name for line in overexp_mutants])
        the_multiples = set([DictyMutant(line).name for line in multiple_mutants])
        the_develops = set([DictyMutant(line).name for line in develop_mutants])
        the_others = set([DictyMutant(line).name for line in other_mutants])

        for mutant in _mutants:
            if mutant.name in the_nulls: mutant.null = True
            if mutant.name in the_overexps: mutant.overexp = True 
            if mutant.name in the_multiples: mutant.multiple = True
            if mutant.name in the_develops: mutant.develop = True
            if mutant.name in the_others: mutant.other = True
       
        final_mutants = {x: x for x in _mutants}
        return final_mutants

    def pickle_data(self):
        return pickle.dumps(self.download_mutants(), -1)

    @classmethod
    def get_instance(cls):
        if not hasattr(cls, "_shared_dict"):
            dicty = DictyMutants()
            cls._shared_dict = dicty.__dict__
        instance = DictyMutants.__new__(DictyMutants)
        instance.__dict__ = cls._shared_dict
        return instance

    def mutants(self):
        return self._mutants.keys()

    def genes(self):
        return sorted(set(reduce(list.__add__,
                                 [self.mutant_genes(mutant)
                                  for mutant in self.mutants()], [])))

    def phenotypes(self):
        return sorted(set(reduce(list.__add__,
                                 [self.mutant_phenotypes(mutant)
                                  for mutant in self.mutants()], [])))

    def mutant_genes(self, mutant):
        return self._mutants[mutant].genes
    
    def mutant_phenotypes(self, mutant):
        return self._mutants[mutant].phenotypes

    def gene_mutants(self):
        dgm = defaultdict(set)
        for mutant, genes in [(mutant, self.mutant_genes(mutant))
                              for mutant in self.mutants()]:
            for gene in genes:
                dgm[gene].add(mutant)
        return dgm

    def phenotype_mutants(self):
        dpm = defaultdict(set)
        for mutant, phenotypes in [(mutant, self.mutant_phenotypes(mutant))
                                   for mutant in self.mutants()]:
            for phenotype in phenotypes:
                dpm[phenotype].add(mutant)
        return dpm


def mutants():
    """ Return all :obj:`DictyMutant` objects.
    """
    return DictyMutants.get_instance().mutants()


def genes():
    """ Return a set of all genes referenced in the Dictybase.
    """
    return DictyMutants.get_instance().genes()


def phenotypes():
    """ Return a set of all phenotypes referenced in Dictybase.
    """
    return DictyMutants.get_instance().phenotypes()


def mutant_genes(mutant):
    """ Return a set of all genes referenced by a `mutant` in Dictybase.
    """
    return DictyMutants.get_instance().mutant_genes(mutant)


def mutant_phenotypes(mutant):   
    """ Return a set of all phenotypes referenced by a `mutant` in Dictybase.
    """
    return DictyMutants.get_instance().mutant_phenotypes(mutant)


def gene_mutants():
    """ Return a dictionary { gene: set(mutant_objects for mutant), ... }.
    """
    return DictyMutants.get_instance().gene_mutants()


def phenotype_mutants():
    """ Return a dictionary { phenotype: set(mutant_objects for mutant), ... }.
    """
    return DictyMutants.get_instance().phenotype_mutants()


def download_mutants():
    return DictyMutants.get_instance().pickle_data()


if __name__ == "__main__":
    dicty_mutants = mutants()
    print mutant_phenotypes(dicty_mutants[0])



"""<name>GEO Data Sets</name>
<description>Access to Gene Expression Omnibus data sets.</description>
<priority>20</priority>
<contact>Ales Erjavec (ales.erjavec(@at@)fri.uni-lj.si)</contact>
<icon>icons/GEODataSets.svg</icon>
"""

from __future__ import absolute_import, with_statement

import sys
import os
import glob
import string
import urllib2
from collections import defaultdict
from functools import partial

from Orange.utils import lru_cache
from Orange.utils import serverfiles
from Orange.orng.orngDataCaching import data_hints
from Orange.OrangeWidgets import OWGUI, OWGUIEx
from Orange.OrangeWidgets.OWWidget import *

from Orange.OrangeWidgets.OWConcurrent import (
    ThreadExecutor, Task, methodinvoke
)

from .. import geo

NAME = "GEO Data Sets"
DESCRIPTION = "Access to Gene Expression Omnibus data sets."
ICON = "icons/GEODataSets.svg"
PRIORITY = 20

INPUTS = []
OUTPUTS = [("Expression Data", Orange.data.Table)]

REPLACES = ["_bioinformatics.widgets.OWGEODatasets.OWGEODatasets"]


TextFilterRole = OWGUI.OrangeUserRole.next()


class MySortFilterProxyModel(QSortFilterProxyModel):
    def __init__(self, parent=None):
        QSortFilterProxyModel.__init__(self, parent)
        self._filter_strings = []
        self._cache = {}
        self._cache_fixed = {}
        self._cache_prefix = {}
        self._row_text = {}

        # Create a cached version of _filteredRows
        self._filteredRows = lru_cache(100)(self._filteredRows)

    def setSourceModel(self, model):
        """Set the source model for the filter.
        """
        self._filter_strings = []
        self._cache = {}
        self._cache_fixed = {}
        self._cache_prefix = {}
        self._row_text = {}
        QSortFilterProxyModel.setSourceModel(self, model)

    def addFilterFixedString(self, string, invalidate=True):
        """ Add `string` filter to the list of filters. If invalidate is
        True the filter cache will be recomputed.
        """
        self._filter_strings.append(string)
        all_rows = range(self.sourceModel().rowCount())
        row_text = [self.rowFilterText(row) for row in all_rows]
        self._cache[string] = [string in text for text in row_text]
        if invalidate:
            self.updateCached()
            self.invalidateFilter()

    def removeFilterFixedString(self, index=-1, invalidate=True):
        """ Remove the `index`-th filter string. If invalidate is True the
        filter cache will be recomputed.
        """
        string = self._filter_strings.pop(index)
        del self._cache[string]
        if invalidate:
            self.updateCached()
            self.invalidate()

    def setFilterFixedStrings(self, strings):
        """Set a list of string to be the new filters.
        """
        to_remove = set(self._filter_strings) - set(strings)
        to_add = set(strings) - set(self._filter_strings)
        for str in to_remove:
            self.removeFilterFixedString(
                self._filter_strings.index(str),
                invalidate=False)

        for str in to_add:
            self.addFilterFixedString(str, invalidate=False)
        self.updateCached()
        self.invalidate()

    def _filteredRows(self, filter_strings):
        """Return a dictionary mapping row indexes to True False values.

        .. note:: This helper function is wrapped in the __init__ method.

        """
        all_rows = range(self.sourceModel().rowCount())
        cache = self._cache
        return dict([(row, all([cache[str][row] for str in filter_strings]))
                     for row in all_rows])

    def updateCached(self):
        """Update the combined filter cache.
        """
        self._cache_fixed = self._filteredRows(
            tuple(sorted(self._filter_strings)))

    def setFilterFixedString(self, string):
        """Should this raise an error? It is not being used.
        """
        QSortFilterProxyModel.setFilterFixedString(self, string)

    def rowFilterText(self, row):
        """Return text for `row` to filter on.
        """
        f_role = self.filterRole()
        f_column = self.filterKeyColumn()
        s_model = self.sourceModel()
        data = s_model.data(s_model.index(row, f_column), f_role)
        if isinstance(data, QVariant):
            data = unicode(data.toString(), errors="ignore")
        else:
            data = unicode(data, errors="ignore")
        return data

    def filterAcceptsRow(self, row, parent):
        return self._cache_fixed.get(row, True)

    def lessThan(self, left, right):
        # TODO: Remove fixed column handling
        if left.column() == 1 and right.column():
            left_gds = str(left.data(Qt.DisplayRole).toString())
            right_gds = str(right.data(Qt.DisplayRole).toString())
            left_gds = left_gds.lstrip("GDS")
            right_gds = right_gds.lstrip("GDS")
            try:
                return int(left_gds) < int(right_gds)
            except ValueError:
                pass
        return QSortFilterProxyModel.lessThan(self, left, right)

from Orange.OrangeWidgets.OWGUI import LinkStyledItemDelegate, LinkRole


def childiter(item):
    """ Iterate over the children of an QTreeWidgetItem instance.
    """
    for i in range(item.childCount()):
        yield item.child(i)


class OWGEODatasets(OWWidget):
    settingsList = ["outputRows", "mergeSpots", "gdsSelectionStates",
                    "splitterSettings", "currentGds", "autoCommit",
                    "datasetNames"]

    def __init__(self, parent=None, signalManager=None, name=" GEO Data Sets"):
        OWWidget.__init__(self, parent, signalManager, name)

        self.outputs = [("Expression Data", ExampleTable)]

        ## Settings
        self.selectedAnnotation = 0
        self.includeIf = False
        self.minSamples = 3
        self.autoCommit = False
        self.outputRows = 0
        self.mergeSpots = True
        self.filterString = ""
        self.currentGds = None
        self.selectionChanged = False
        self.autoCommit = False
        self.gdsSelectionStates = {}
        self.splitterSettings = [
            '\x00\x00\x00\xff\x00\x00\x00\x00\x00\x00\x00\x02\x00\x00\x01\xea\x00\x00\x00\xd7\x01\x00\x00\x00\x07\x01\x00\x00\x00\x02',
            '\x00\x00\x00\xff\x00\x00\x00\x00\x00\x00\x00\x02\x00\x00\x01\xb5\x00\x00\x02\x10\x01\x00\x00\x00\x07\x01\x00\x00\x00\x01'
        ]
        self.datasetNames = {}
        self.loadSettings()

        self.datasetName = ""

        ## GUI
        self.infoBox = OWGUI.widgetLabel(
            OWGUI.widgetBox(self.controlArea, "Info", addSpace=True),
            "Initializing\n\n"
        )

        box = OWGUI.widgetBox(self.controlArea, "Output", addSpace=True)
        OWGUI.radioButtonsInBox(box, self, "outputRows",
                                ["Genes or spots", "Samples"], "Rows",
                                callback=self.commitIf)
        OWGUI.checkBox(box, self, "mergeSpots", "Merge spots of same gene",
                       callback=self.commitIf)

        OWGUI.separator(box)
        self.nameEdit = OWGUI.lineEdit(
            box, self, "datasetName", "Data set name",
            tooltip="Override the default output data set name",
            callback=self.onNameEdited
        )
        self.nameEdit.setPlaceholderText("")

        box = OWGUI.widgetBox(self.controlArea, "Commit", addSpace=True)
        self.commitButton = OWGUI.button(box, self, "Commit",
                                         callback=self.commit)
        cb = OWGUI.checkBox(box, self, "autoCommit", "Commit on any change")
        OWGUI.setStopper(self, self.commitButton, cb, "selectionChanged",
                         self.commit)
        OWGUI.rubber(self.controlArea)

        self.filterLineEdit = OWGUIEx.lineEditHint(
            self.mainArea, self, "filterString", "Filter",
            caseSensitive=False, matchAnywhere=True,
            callback=self.filter,  delimiters=" ")

        splitter = QSplitter(Qt.Vertical, self.mainArea)
        self.mainArea.layout().addWidget(splitter)
        self.treeWidget = QTreeView(splitter)

        self.treeWidget.setSelectionMode(QAbstractItemView.SingleSelection)
        self.treeWidget.setRootIsDecorated(False)
        self.treeWidget.setSortingEnabled(True)
        self.treeWidget.setAlternatingRowColors(True)
        self.treeWidget.setUniformRowHeights(True)
        self.treeWidget.setEditTriggers(QTreeView.NoEditTriggers)

        linkdelegate = LinkStyledItemDelegate(self.treeWidget)
        self.treeWidget.setItemDelegateForColumn(1, linkdelegate)
        self.treeWidget.setItemDelegateForColumn(8, linkdelegate)
        self.treeWidget.setItemDelegateForColumn(
            0, OWGUI.IndicatorItemDelegate(self.treeWidget,
                                           role=Qt.DisplayRole))

        proxyModel = MySortFilterProxyModel(self.treeWidget)
        self.treeWidget.setModel(proxyModel)
        self.treeWidget.selectionModel().selectionChanged.connect(
            self.updateSelection
        )
        self.treeWidget.viewport().setMouseTracking(True)

        splitterH = QSplitter(Qt.Horizontal, splitter)

        box = OWGUI.widgetBox(splitterH, "Description")
        self.infoGDS = OWGUI.widgetLabel(box, "")
        self.infoGDS.setWordWrap(True)
        OWGUI.rubber(box)

        box = OWGUI.widgetBox(splitterH, "Sample Annotations")
        self.annotationsTree = QTreeWidget(box)
        self.annotationsTree.setHeaderLabels(
            ["Type (Sample annotations)", "Sample count"]
        )
        self.annotationsTree.setRootIsDecorated(True)
        box.layout().addWidget(self.annotationsTree)
        self.annotationsTree.itemChanged.connect(
            self.annotationSelectionChanged
        )
        self._annotationsUpdating = False
        self.splitters = splitter, splitterH

        for sp, setting in zip(self.splitters, self.splitterSettings):
            sp.splitterMoved.connect(self.splitterMoved)
            sp.restoreState(setting)

        self.searchKeys = ["dataset_id", "title", "platform_organism",
                           "description"]

        self.gds = []
        self.gds_info = None

        self.resize(1000, 600)

        self.setBlocking(True)
        self.setEnabled(False)
        self.progressBarInit()

        self._executor = ThreadExecutor()

        func = partial(get_gds_model,
                       methodinvoke(self, "_setProgress", (float,)))
        self._inittask = Task(function=func)
        self._inittask.finished.connect(self._initializemodel)
        self._executor.submit(self._inittask)

        self._datatask = None

    @pyqtSlot(float)
    def _setProgress(self, value):
        self.progressBarValue = value

    def _initializemodel(self):
        assert self.thread() is QThread.currentThread()
        model, self.gds_info, self.gds = self._inittask.result()
        model.setParent(self)

        proxy = self.treeWidget.model()
        proxy.setFilterKeyColumn(0)
        proxy.setFilterRole(TextFilterRole)
        proxy.setFilterCaseSensitivity(False)
        proxy.setFilterFixedString(self.filterString)

        proxy.setSourceModel(model)
        proxy.sort(0, Qt.DescendingOrder)

        self.progressBarFinished()
        self.setBlocking(False)
        self.setEnabled(True)

        filter_items = " ".join(
            gds[key] for gds in self.gds for key in self.searchKeys
        )
        tr_chars = ",.:;!?(){}[]_-+\\|/%#@$^&*<>~`"
        tr_table = string.maketrans(tr_chars, " " * len(tr_chars))
        filter_items = filter_items.translate(tr_table)

        filter_items = sorted(set(filter_items.split(" ")))
        filter_items = [item for item in filter_items if len(item) > 3]
        self.filterLineEdit.setItems(filter_items)

        if self.currentGds:
            gdss = [(i, proxy.data(proxy.index(i, 1), Qt.DisplayRole))
                    for i in range(proxy.rowCount())]
            current = [i for i, variant in gdss
                       if variant.isValid() and
                       str(variant.toString()) == self.currentGds["dataset_id"]]
            if current:
                current_index = proxy.index(current[0], 0)
                self.treeWidget.selectionModel().select(
                    current_index,
                    QItemSelectionModel.Select |
                    QItemSelectionModel.Rows
                )
                self.treeWidget.scrollTo(current_index)

        for i in range(8):
            self.treeWidget.resizeColumnToContents(i)

        self.treeWidget.setColumnWidth(
            1, min(self.treeWidget.columnWidth(1), 300))
        self.treeWidget.setColumnWidth(
            2, min(self.treeWidget.columnWidth(2), 200))

        self.updateInfo()

    def updateInfo(self):
        gds_info = self.gds_info
        text = ("%i datasets\n%i datasets cached\n" %
                (len(gds_info),
                 len(glob.glob(serverfiles.localpath("GEO") + "/GDS*"))))
        filtered = self.treeWidget.model().rowCount()
        if len(self.gds) != filtered:
            text += ("%i after filtering") % filtered
        self.infoBox.setText(text)

    def updateSelection(self, *args):
        current = self.treeWidget.selectedIndexes()
        mapToSource = self.treeWidget.model().mapToSource
        current = [mapToSource(index).row() for index in current]
        if current:
            self.currentGds = self.gds[current[0]]
            self.setAnnotations(self.currentGds)
            self.infoGDS.setText(self.currentGds.get("description", ""))
            self.nameEdit.setPlaceholderText(self.currentGds["title"])
            self.datasetName = \
                self.datasetNames.get(self.currentGds["dataset_id"], "")
        else:
            self.currentGds = None
            self.nameEdit.setPlaceholderText("")
            self.datasetName = ""

        self.commitIf()

    def setAnnotations(self, gds):
        self._annotationsUpdating = True
        self.annotationsTree.clear()

        annotations = defaultdict(set)
        subsetscount = {}
        for desc in gds["subsets"]:
            annotations[desc["type"]].add(desc["description"])
            subsetscount[desc["description"]] = str(len(desc["sample_id"]))

        for type, subsets in annotations.items():
            key = (gds["dataset_id"], type)
            subsetItem = QTreeWidgetItem(self.annotationsTree, [type])
            subsetItem.setFlags(subsetItem.flags() | Qt.ItemIsUserCheckable |
                                Qt.ItemIsTristate)
            subsetItem.setCheckState(
                0, self.gdsSelectionStates.get(key, Qt.Checked)
            )
            subsetItem.key = key
            for subset in subsets:
                key = (gds["dataset_id"], type, subset)
                item = QTreeWidgetItem(
                    subsetItem, [subset, subsetscount.get(subset, "")]
                )
                item.setFlags(item.flags() | Qt.ItemIsUserCheckable)
                item.setCheckState(
                    0, self.gdsSelectionStates.get(key, Qt.Checked)
                )
                item.key = key
        self._annotationsUpdating = False
        self.annotationsTree.expandAll()
        for i in range(self.annotationsTree.columnCount()):
            self.annotationsTree.resizeColumnToContents(i)

    def annotationSelectionChanged(self, item, column):
        if self._annotationsUpdating:
            return
        for i in range(self.annotationsTree.topLevelItemCount()):
            item = self.annotationsTree.topLevelItem(i)
            self.gdsSelectionStates[item.key] = item.checkState(0)
            for j in range(item.childCount()):
                child = item.child(j)
                self.gdsSelectionStates[child.key] = child.checkState(0)

    def filter(self):
        filter_string = unicode(self.filterLineEdit.text(), errors="ignore")
        proxyModel = self.treeWidget.model()
        if proxyModel:
            strings = filter_string.lower().strip().split()
            proxyModel.setFilterFixedStrings(strings)
            self.updateInfo()

    def selectedSamples(self):
        """
        Return the currently selected sample annotations.

        The return value is a list of selected (sample type, sample value)
        tuples.

        .. note:: if some Sample annotation type has no selected values.
                  this method will return all values for it.

        """
        samples = []
        unused_types = []
        used_types = []
        for stype in childiter(self.annotationsTree.invisibleRootItem()):
            selected_values = []
            all_values = []
            for sval in childiter(stype):
                value = (str(stype.text(0)), str(sval.text(0)))
                if self.gdsSelectionStates.get(sval.key, True):
                    selected_values.append(value)
                all_values.append(value)
            if selected_values:
                samples.extend(selected_values)
                used_types.append(str(stype.text(0)))
            else:
                # If no sample of sample type is selected we don't filter
                # on it.
                samples.extend(all_values)
                unused_types.append(str(stype.text(0)))

        return samples, used_types

    def commitIf(self):
        if self.autoCommit:
            self.commit()
        else:
            self.selectionChanged = True

    def commit(self):
        if self.currentGds:
            self.error(0)
            sample_type = None
            self.progressBarInit()
            self.progressBarSet(10)

            _, groups = self.selectedSamples()
            if len(groups) == 1 and self.outputRows:
                sample_type = groups[0]

            self.setEnabled(False)
            self.setBlocking(True)

            def get_data(gds_id, report_genes, transpose, sample_type, title):
                gds = geo.GDS(gds_id)
                data = gds.getdata(
                    report_genes=report_genes, transpose=transpose,
                    sample_type=sample_type
                )
                data.name = title
                return data

            get_data = partial(
                get_data, self.currentGds["dataset_id"],
                report_genes=self.mergeSpots,
                transpose=self.outputRows,
                sample_type=sample_type,
                title=self.datasetName or self.currentGds["title"]
            )
            self._datatask = Task(function=get_data)
            self._datatask.finished.connect(self._on_dataready)
            self._executor.submit(self._datatask)

    def _on_dataready(self):
        self.setEnabled(True)
        self.setBlocking(False)

        self.progressBarSet(50)

        try:
            data = self._datatask.result()
        except urllib2.URLError as error:
            self.error(0, "Error while connecting to the NCBI ftp server! %r" %
                       error)
            self._datatask = None
            self.progressBarFinished()
            return

        self._datatask = None

        data_name = data.name
        samples, _ = self.selectedSamples()

        self.warning(0)
        message = None
        if self.outputRows:
            def samplesinst(ex):
                out = []
                for i, a in data.domain.get_metas().items():
                    out.append((a.name, ex[i].value))
                if data.domain.class_var.name != 'class':
                    out.append((data.domain.class_var.name, ex[-1].value))
                return out
            samples = set(samples)

            select = [1 if samples.issuperset(samplesinst(ex)) else 0
                      for ex in data]
            data = data.select(select)
            if len(data) == 0:
                message = "No samples with selected sample annotations."
        else:
            samples = set(samples)
            domain = orange.Domain(
                [attr for attr in data.domain.attributes
                 if samples.issuperset(attr.attributes.items())],
                data.domain.classVar
            )
            domain.addmetas(data.domain.getmetas())
            if len(domain.attributes) == 0:
                message = "No samples with selected sample annotations."
            stypes = set(s[0] for s in samples)
            for attr in domain.attributes:
                attr.attributes = dict(
                    (key, value) for key, value in attr.attributes.items()
                    if key in stypes
                )
            data = orange.ExampleTable(domain, data)

        if message is not None:
            self.warning(0, message)

        data_hints.set_hint(data, "taxid", self.currentGds.get("taxid", ""),
                            10.0)
        data_hints.set_hint(data, "genesinrows", self.outputRows, 10.0)

        self.progressBarFinished()
        data.name = data_name
        self.send("Expression Data", data)

        model = self.treeWidget.model().sourceModel()
        row = self.gds.index(self.currentGds)

        model.setData(model.index(row, 0),  QVariant(" "), Qt.DisplayRole)

        self.updateInfo()
        self.selectionChanged = False

    def splitterMoved(self, *args):
        self.splitterSettings = [str(sp.saveState()) for sp in self.splitters]

    def onDeleteWidget(self):
        if self._inittask:
            self._inittask.future().cancel()
            self._inittask.finished.disconnect(self._initializemodel)
        if self._datatask:
            self._datatask.future().cancel()
            self._datatask.finished.disconnect(self._on_dataready)
        self._executor.shutdown(wait=False)

        super(OWGEODatasets, self).onDeleteWidget()

    def onNameEdited(self):
        if self.currentGds:
            gds_id = self.currentGds["dataset_id"]
            self.datasetNames[gds_id] = unicode(self.nameEdit.text())
            self.commitIf()

def get_gds_model(progress=lambda val: None):
    """
    Initialize and return a GDS datasets model.

    :param progress: A progress callback.
    :rval tuple:
        A tuple of (QStandardItemModel, geo.GDSInfo, [geo.GDS])

    .. note::
        The returned QStandardItemModel's thread affinity is set to
        the GUI thread.

    """
    progress(1)
    info = geo.GDSInfo()
    search_keys = ["dataset_id", "title", "platform_organism", "description"]
    cache_dir = serverfiles.localpath(geo.DOMAIN)
    gds_link = "http://www.ncbi.nlm.nih.gov/sites/GDSbrowser?acc={0}"
    pm_link = "http://www.ncbi.nlm.nih.gov/pubmed/{0}"
    gds_list = []

    def is_cached(gds):
        return os.path.exists(os.path.join(cache_dir, gds["dataset_id"]) +
                              ".soft.gz")

    def item(displayvalue, item_values={}):
        item = QStandardItem()
        item.setData(displayvalue, Qt.DisplayRole)
        for role, value in item_values.iteritems():
            item.setData(value, role)
        return item

    def gds_to_row(gds):
        #: Text for easier full search.
        search_text = unicode(
            " | ".join([gds.get(key, "").lower()
                        for key in search_keys]),
            errors="ignore"
        )
        row = [
            item(" " if is_cached(gds) else "",
                 {TextFilterRole: search_text}),
            item(gds["dataset_id"],
                 {LinkRole: gds_link.format(gds["dataset_id"])}),
            item(gds["title"]),
            item(gds["platform_organism"]),
            item(len(gds["samples"])),
            item(gds["feature_count"]),
            item(gds["gene_count"]),
            item(len(gds["subsets"])),
            item(gds.get("pubmed_id", ""),
                 {LinkRole: pm_link.format(gds["pubmed_id"])
                            if gds.get("pubmed_id")
                            else QVariant()})
        ]
        return row

    model = QStandardItemModel()
    model.setHorizontalHeaderLabels(
        ["", "ID", "Title", "Organism", "Samples", "Features",
         "Genes", "Subsets", "PubMedID"]
    )
    progress(20)
    for gds in info.values():
        model.appendRow(gds_to_row(gds))

        gds_list.append(gds)

    progress(50)

    if QThread.currentThread() is not QCoreApplication.instance().thread():
        model.moveToThread(QCoreApplication.instance().thread())
    return model, info, gds_list


if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = OWGEODatasets()
    w.show()
    app.exec_()
    w.saveSettings()



"""
Return types from api classes interface for the SOAP kegg api.

"""

from datetime import datetime
from collections import namedtuple
from operator import methodcaller

Definition = namedtuple(
    "Definition",
    ["entry_id",
     "definition"]
)


def _Definition_from_items(items):
    """ Definition 'items' tuple from a list of items
    """
    items_list = []
    for name, val in items:
        if isinstance(name, list):
            name = name[0]
        if isinstance(val, list):
            val = val[0]
        items_list.append((str(name), str(val)))
    return Definition(**dict(items_list))


def _Definition_from_str(text):
    """
    Return a `Definition` item by parsing a tab separated string `text`
    i.e. text must be of the form '<entry_id>\t<definition>'

    """
    return Definition(*text.split("\t", 1))


Definition.from_items = staticmethod(_Definition_from_items)
Definition.from_str = staticmethod(_Definition_from_str)


OrganismSummary = namedtuple(
    "OrganismSummary",
    ["entry_id",
     "org_code",
     "name",
     "lineage"],
)


def OrganismSummary_from_str(string):
    return OrganismSummary(*string.split("\t"))

OrganismSummary.from_str = staticmethod(OrganismSummary_from_str)


BInfo = namedtuple(
    'BInfo',
    ["entry_id",
    "definition",
    "name",
    "release",
    "curator",
    "contents",
    "last_update",
    "supported_formats"]
)


def _BInfo_from_text(text):
    """ Parse the return string from info into a new BInfo instance.
    """
    lines = text.splitlines()
    name, definition = lines[0].split(" ", 1)
    definition = definition.strip()
    entry_id, release = lines[1].split(" ", 1)
    _, release = release.strip().split(" ", 1)
    curator = lines[2].strip()
    contents = "\n".join(map(methodcaller("strip"), lines[3:]))

    return BInfo(entry_id, definition, name, release, curator,
                 contents, None, None)

BInfo.from_text = staticmethod(_BInfo_from_text)


Link = namedtuple("Link", ["entry_id1", "entry_id2"])


SSDBRelation = namedtuple(
    "SSDBRelation", ["genes_id1",
                     "genes_id2",
                     "sw_score",
                     "bit_score",
                     "identity",
                     "overlap",
                     "start_position1",
                     "end_position1",
                     "start_position2",
                     "end_position2",
                     "best_flag_1to2",
                     "best_flag_2to1",
                     "definition1",
                     "definition2",
                     "length1",
                     "length2"
                     ])

MotifResult = namedtuple(
    "MotifResult", ["motif_id",
                    "definition",
                    "genes_id",
                    "start_position",
                    "end_position",
                    "score",
                    "evalue"
                    ])

LinkDBRelation = namedtuple(
    "LinkDBRelation", ["entry_id1",
                       "entry_id2",
                       "type",
                       "path"
                       ])

PathwayElement = namedtuple(
    "PathwayElement", ["element_id",
                       "type",
                       "names",
                       "components"
                       ])

PathwayElementRelation = namedtuple(
    "PathwayElementRelation", ["element_id1",
                               "element_id2",
                               "type",
                               "subtypes"
                               ])

Subtype = namedtuple(
    "Subtype", ["element_id",
                "relation",
                "type",
                ])



"""
Print out some information on specific GEO's data set.
Does not download the data set.
"""

import Orange.bio.geo
import textwrap

gdsinfo = Orange.bio.geo.GDSInfo()
gds = gdsinfo["GDS10"]

print "ID:", gds["dataset_id"]
print "Features:", gds["feature_count"]
print "Genes:", gds["gene_count"]
print "Organism:", gds["platform_organism"]
print "PubMed ID:", gds["pubmed_id"]
print "Sample types:"
for sampletype in set([sinfo["type"] for sinfo in gds["subsets"]]):
    ss = [sinfo["description"] for sinfo in gds["subsets"] if sinfo["type"]==sampletype]
    print "  %s (%s)" % (sampletype, ", ".join(ss))
print
print "Description:"
print "\n".join(textwrap.wrap(gds["description"], 70))



from __future__ import absolute_import, with_statement

import cPickle as pickle, os, tempfile, sys
from collections import defaultdict
import datetime

import Orange.core as orange
from Orange.orng import orngServerFiles

from .. import go as obiGO, kegg as obiKEGG, taxonomy as obiTaxonomy
from .. import dicty 
obiDictyMutants = dicty.phenotypes
from .. import omim as obiOMIM
from .. import go as obiGO

from . import transform

sfdomain = "gene_sets"

def nth(l,n):
    return [ a[n] for a in l]

class NoGenesetsException(Exception): pass

def goGeneSets(org):
    """Returns gene sets from GO."""
    ontology = obiGO.Ontology()
    annotations = obiGO.Annotations(org, ontology=ontology)

    genesets = []
    link_fmt = "http://amigo.geneontology.org/cgi-bin/amigo/term-details.cgi?term=%s"
    for termn, term in ontology.terms.items():
        genes = annotations.GetAllGenes(termn)
        hier = ("GO", term.namespace)
        if len(genes) > 0:
            gs = GeneSet(id=termn, name=term.name, genes=genes, hierarchy=hier, organism=org, link=link_fmt % termn)
            genesets.append(gs)

    return GeneSets(genesets)

def strornone(x):
    return str(x) if x != None else x

def keggGeneSets(org):
    """
    Returns gene sets from KEGG pathways.
    """

    kegg = obiKEGG.KEGGOrganism(org)

    genesets = []
    for id in kegg.pathways():
        print id
        pway = obiKEGG.KEGGPathway(id)
        hier = ("KEGG","pathways")
        gs = GeneSet(id=id,
                                 name=pway.title,
                                 genes=kegg.get_genes_by_pathway(id),
                                 hierarchy=hier,
                                 organism=org,
                                 link=pway.link)
        genesets.append(gs)

    return GeneSets(genesets)

def dictyMutantSets():
    """
    Return dicty mutant phenotype gene sets from Dictybase
    """
    link_fmt = "http://dictybase.org/db/cgi-bin/dictyBase/SC/scsearch.pl?searchdb=strains&search_term=%s&column=all&B1=Submit" 
    #genesets = [GeneSet(id=mutant.name, name=mutant.descriptor, genes=obiDictyMutants.mutant_genes(mutant), hierarchy=("Dictybase", "Mutants"), organism="352472", # 352472 gathered from obiGO.py code_map -> Dicty identifier
    #                    link=(link_fmt % mutant.name if mutant.name else None)) \
    #                    for mutant in obiDictyMutants.mutants()]
 
    genesets = [GeneSet(id=phenotype, name=phenotype, genes=[obiDictyMutants.mutant_genes(mutant)[0] for mutant in mutants], hierarchy=("Dictybase", "Phenotypes"), organism="352472", # 352472 gathered from obiGO.py code_map -> Dicty identifier
                        link="") \
                        for phenotype, mutants in obiDictyMutants.phenotype_mutants().items()]

    return GeneSets(genesets)

def cytobandGeneSets():
    """
    Create cytoband gene sets from Stanford Microarray Database
    """
    import urllib2

    url = "http://www-stat.stanford.edu/~tibs/GSA/cytobands-stanford.gmt"
    stream = urllib2.urlopen(url)
    data = stream.read().splitlines()

    genesets = []
    for band in data:
        b = band.split("\t")
        genesets.append(GeneSet(id=b[0], name=b[1], genes=b[2:] if b[2:] else [], hierarchy=("Cytobands",), organism="9606", link=""))          

    return GeneSets(genesets)

def reactomePathwaysGeneSets():
    """
    Prepare human pathways gene sets from reactome pathways
    """
    import urllib
    import io
    from zipfile import ZipFile

    url = urllib.urlopen("http://www.reactome.org/download/current/ReactomePathways.gmt.zip")
    memfile = io.BytesIO(url.read())
    with ZipFile(memfile, "r") as myzip:
        f = myzip.open("ReactomePathways.gmt")
        content = f.read().splitlines()      

    genesets = [GeneSet(id=path.split("\t")[0], name=path.split("\t")[0], genes=path.split("\t")[2:] if path.split("\t")[2:] else [], hierarchy=("Reactome", "Pathways"), organism="9606", link="") for path in content]
    return GeneSets(genesets)


def omimGeneSets():
    """
    Return gene sets from OMIM (Online Mendelian Inheritance in Man) diseses
    """
    genesets = [GeneSet(id=disease.id, name=disease.name, genes=obiOMIM.disease_genes(disease), hierarchy=("OMIM",), organism="9606",
                    link=("http://www.omim.org/entry/%s" % disease.id if disease.id else None)) \
                    for disease in obiOMIM.diseases()]
    return GeneSets(genesets)

def miRNAGeneSets(org):
    """
    Return gene sets from miRNA targets
    """
    from .. import obimiRNA
    org_code = obiKEGG.from_taxid(org)
    link_fmt = "http://www.mirbase.org/cgi-bin/mirna_entry.pl?acc=%s"
    mirnas = [(id, obimiRNA.get_info(id)) for id in obimiRNA.ids(org_code)]
    genesets = [GeneSet(id=mirna.matACC, name=mirna.matID, genes=mirna.targets.split(","), hierarchy=("miRNA", "Targets"),
                        organism=org, link=link_fmt % mirna.matID) for id, mirna in mirnas]
    return GeneSets(genesets)

def go_miRNASets(org, ontology=None, enrichment=True, pval=0.05, treshold=0.04):
    from .. import obimiRNA
    mirnas = obimiRNA.ids(int(org))
    if ontology is None:
        ontology = obiGO.Ontology()

    annotations = obiGO.Annotations(org, ontology=ontology)

    go_sets = obimiRNA.get_GO(mirnas, annotations, enrichment=enrichment, pval=pval, goSwitch=False)

    go_sets = obimiRNA.filter_GO(go_sets, annotations, treshold=treshold)

    link_fmt = "http://amigo.geneontology.org/cgi-bin/amigo/term-details.cgi?term=%s"
    gsets = [GeneSet(id=key, name=ontology[key].name, genes=value, hierarchy=("miRNA", "go_sets",),
                        organism=org, link=link_fmt % key) for key, value in go_sets.items()]
    gset = GeneSets(gsets)
    return gset

import re

linkre = re.compile("(.*?)\s?(?:\[(https?://[^[^\]]*)\])?$")

def loadGMT(contents, name):
    """
    Eech line consists of tab separated elements. First is
    the geneset name, next is it's description.

    For now the description is skipped.

    Example Gene Set (.gmt) file:
    anti-liver_sw   anti-liver_sw   BMPR1A  APOD    WSB1    BMI1    SLC2A1  ...
    B-cells_sw  B-cells_sw  E2F5    NCF1    PALM2-AKAP2 IRF4    SLC2A1  ...
    Bladder_sw  Bladder_sw  PLCD4   ANGPTL1 LOC286191   ST0N1   LOC283904   ...
    cerebellum_sw   cerebellum_sw   C19orf43    LOC653464   KI110802    ...
    Cervix_sw   Cervix_sw   LAMA4   GSTM5   SNX19   DKK1    NT5E    ...
    """

    def hline(s):
        tabs = [tab.strip() for tab in s.split("\t")]
        groups = linkre.match(tabs[1]).groups()
        return GeneSet(id=tabs[0], description=groups[0], link=groups[1],
                                   hierarchy=("Custom",name), genes=tabs[2:])

    def handleNELines(s, fn):
        """
        Run function on nonempty lines of a string.
        Return a list of results for each line.
        """
        lines = (l.strip() for l in s.splitlines())
        return [fn(l) for l in lines if l]

    return GeneSets(handleNELines(contents, hline))

def getGenesetsStats(genesets):
    num_sets = len(genesets)
    unique_genes = len(set([gene for geneset in genesets for gene in geneset.genes]))
    genes_per_geneset = sum([len(geneset.genes) for geneset in genesets])/num_sets
    return num_sets, unique_genes, genes_per_geneset
"""
We have multiple paths for gene set data:
buffer/bigfiles/gene_sets
and
buffer/gene_sets_local
both have available.txt
"""

def omakedirs(dir):
    try:
        os.makedirs(dir)
    except OSError:
        pass

def local_path():
    """ Returns local path for gene sets. Creates it if it does not exists
    yet. """
    from Orange.orng import orngEnviron
    pth = os.path.join(orngEnviron.directoryNames["bufferDir"], "gene_sets_local")
    omakedirs(pth)
    return pth

def build_index(dir):
    """ Returns gene set availability index for some folder. """
    pass

def filename(hierarchy, organism):
    """ Obtain a filename for given hierarchy and organism. """
    return "gs_" + "_._".join(hierarchy + \
        (organism if organism != None else "",)) + ".pck"

def filename_parse(fn):
    """ Returns a hierarchy and the organism from the filename."""
    fn = fn[3:-4]
    parts = fn.split("_._")
    hierarchy = tuple(parts[:-1])
    org = parts[-1] if parts[-1] != "" else None
    return hierarchy, org

def is_genesets_file(fn):
    return fn.startswith("gs_") and fn.endswith(".pck")

def list_local():
    """ Returns available gene sets from the local repository:
    a list of (hierarchy, organism, on_local) """
    pth = local_path()
    gs_files = filter(is_genesets_file, os.listdir(pth))
    return [ filename_parse(fn) + (True,) for fn in gs_files ]

def remove_local(gene_set):
    """ Removes a given gene set from the local repository. """
    pth = local_path()
    gs_files = filter(is_genesets_file, os.listdir(pth)) 
    for setfile in gs_files:
        if setfile.__contains__(gene_set):
            setBgone = os.path.join(pth, setfile)
            os.remove(setBgone) 

def modification_date(file):
    t = os.path.getmtime(file)
    return datetime.datetime.fromtimestamp(t)

def list_serverfiles_from_flist(flist):
    gs_files = filter(is_genesets_file, flist)
    localfiles = os.listdir(orngServerFiles.localpath(sfdomain))
    localfiles = set(filter(is_genesets_file, localfiles))
    return [ filename_parse(fn) + \
        ((True,) if fn in localfiles else (False,)) for fn in set(gs_files) | localfiles ]

def list_serverfiles_conn(serverfiles=None):
    """ Returns available gene sets from the server files
    repository: a list of (hierarchy, organism, on_local) """
    if serverfiles == None:
        serverfiles = orngServerFiles.ServerFiles()
    flist = serverfiles.listfiles(sfdomain)
    return list_serverfiles_from_flist(flist)

def list_serverfiles():
    fname = orngServerFiles.localpath_download(sfdomain, "index.pck")
    flist = pickle.load(open(fname, 'r'))
    return list_serverfiles_from_flist(flist)

def list_all(org=None, local=None):
    """
    Return gene sets available in the local and ServerFiles repositories. 
    It returns a list of tuples of (hierarchy, organism, available_locally)

    Results can be filtered with the following parameters.

    :param str org: Organism tax id.
    :param bool local: Available locally.
    """
    flist = list_local() + list_serverfiles()
    d = {}
    for h,o,l in flist:
        d[h,o] = min(l, d.get((h,o),True))
    return [ (h,o,l) for (h,o),l in d.items() \
            if (local == None or l == local) and \
               (org == None or o == str(org))
        ]

def update_server_list(serverfiles_upload, serverfiles_list=None):
    if serverfiles_list == None:
        serverfiles_list = orngServerFiles.ServerFiles()

    flist = map(lambda x: filename(*x[:2]), list_serverfiles_conn(serverfiles_list))

    tfname = pickle_temp(flist)

    try:
        fn = "index.pck"
        title = "Gene sets: index"
        tags = [ "gene sets", "index", "essential" ]
        serverfiles_upload.upload(sfdomain, fn, tfname, title, tags)
        serverfiles_upload.unprotect(sfdomain, fn)
    except Exception,e:
        raise e
    finally:
        os.remove(tfname)

def _register_local(genesets):
    """ Registers using the common hierarchy and organism. """
    pth = local_path()
    org = genesets.common_org()
    hierarchy = genesets.common_hierarchy()
    fn = filename(hierarchy, org)

    with open(os.path.join(pth, fn), "w") as f:
        pickle.dump(genesets, f)

    return fn

def pickle_temp(obj):
    """ Pickle a file to a temporary file and returns its name """
    fd,tfname = tempfile.mkstemp()
    os.close(fd)
    f = open(tfname, 'wb')
    pickle.dump(obj, f)
    f.close()
    return tfname

def _register_serverfiles(genesets, serverFiles):
    """ Registers using the common hierarchy and organism. """
    org = genesets.common_org()
    hierarchy = genesets.common_hierarchy()
    fn = filename(hierarchy, org)

    #save to temporary file
    tfname = pickle_temp(genesets)

    try:
        if org != None:
            taxname = obiTaxonomy.name(org)
        title = "Gene sets: " + ", ".join(hierarchy) + \
            ((" (" + taxname + ")") if org != None else "")
        tags = list(hierarchy) + [ "gene sets" ] + ([ taxname ] if org != None else [])  + obiTaxonomy.shortname(org) +\
            ([ "essential" ] if org in obiTaxonomy.essential_taxids() else [] )
        serverFiles.upload(sfdomain, fn, tfname, title, tags)
        serverFiles.unprotect(sfdomain, fn)
    finally:
        os.remove(tfname)

    update_server_list(serverFiles)

def register(genesets, serverFiles=None):
    """ Registers given genesets locally.  The gene set is registered
    by the common hierarchy or organism (None if organisms are different).

    :param GeneSets genesets:
    :param serverFiles: If `serverFiles` is an authenticated ServerFiles connection,
        the input gene sets are uploaded to the repository.  
    """
    if serverFiles == None:
        _register_local(genesets)
    else:
        _register_serverfiles(genesets, serverFiles)

def build_hierarchy_dict(files):
    hierd = defaultdict(list)
    for ind,f in enumerate(files):
        hier, org = f
        for i in range(len(hier)+1):
            hierd[(hier[:i], org)].append(ind)
    return hierd

def load_local(hierarchy, organism):
    return load_fn(hierarchy, organism, list_local, 
        lambda h,o: os.path.join(local_path(), filename(h, o)))

def load_serverfiles(hierarchy, organism):
    return load_fn(hierarchy, organism, list_serverfiles, 
        lambda h,o: orngServerFiles.localpath_download(sfdomain, filename(h, o)))

def load_fn(hierarchy, organism, fnlist, fnget):
    files = map(lambda x: x[:2], fnlist())
    hierd = build_hierarchy_dict(files)
    out = GeneSets()
    matches = hierd[(hierarchy, organism)]
    if not matches:
        exstr = "No gene sets for " + str(hierarchy) + \
                " (org " + str(organism) + ")"
        raise NoGenesetsException(exstr)
    for (h, o) in [ files[i] for i in hierd[(hierarchy, organism)]]:
        fname = fnget(h, o)
        out.update(pickle.load(open(fname, 'r')))
    return out

def load(hierarchy, organism):
    """ First try to load from the local registered folder. If the file
    is not available, load it from the server files. """
    if organism != None:
        try:
            int(organism) #already a taxid
        except:
            organismc = obiTaxonomy.to_taxid(strornone(organism))
            if len(organismc) == 1:
                organism = organismc.pop()
            else:
                exstr = "Could not interpret organism " + str(organism) + \
                      ". Possibilities: " + str(organismc) 
                raise NoGenesetsException(exstr)

    try:
        return load_local(hierarchy, strornone(organism))
    except NoGenesetsException:
        return load_serverfiles(hierarchy, strornone(organism))

def collections(*args):
    """
    Load gene sets from various sources: GMT file, GO, KEGG, and others. 
    Return an instance of :class:`GeneSets`. 
    
    Each arguments specifies a gene set and can be either:
    
    * a filename of a GMT file,
    * a tuple (hierarchy, organism) (for example ``(("KEGG",), "10090")``), or
    * an instance of :class:`GeneSets`
    """
    result = GeneSets()

    for collection in args:
        try:
            result.update(collection)
        except (ValueError, TypeError):
            if issequencens(collection): #have a hierarchy, organism specification
                new = load(*collection)
                result.update(new)
            else:
                if collection.lower()[-4:] == ".gmt": #format from webpage
                    result.update(loadGMT(open(collection,"rt").read(), collection))
                else:
                    raise Exception("collection() accepts files in .gmt format only.")

    return result

def issequencens(x):
    "Is x a sequence and not string ? We say it is if it has a __getitem__ method and it is not an instance of basestring."
    return hasattr(x, '__getitem__') and not isinstance(x, basestring)

class TException(Exception): pass

def upload_genesets(rsf):
    """
    Builds the default gene sets and
    """
    orngServerFiles.update_local_files()

    genesetsfn = [ keggGeneSets, goGeneSets, miRNAGeneSets]
    organisms = obiTaxonomy.common_taxids()
    for fn in genesetsfn:
        for org in organisms:
            try:
                print "Uploading ORG", org, fn
                genesets = fn(org).split_by_hierarchy()
                for gs in genesets:
                    print "registering", gs.common_hierarchy()
                    register(gs, rsf) #server files
                    #register(gs)
                    print "successful", gs.common_hierarchy()
            except obiTaxonomy.UnknownSpeciesIdentifier:
                print "Organism ontology not available", org
            except GenesetRegException:
                print "Empty gene sets.", org


def only_option(a):
    if len(a) == 1:
        return list(a)[0]
    else:
        raise Exception()

class GenesetRegException(Exception): pass

class GeneSet(object):
    """ A single set of genes.
    """

    def __init__(self, genes=[], name=None, id=None, \
        description=None, link=None, organism=None, hierarchy=None, pair=None):
        """
        :param pair: Only for backward compatibility: convert a tuple (name, genes)
            into this object.
        """

        self.hierarchy = hierarchy     
        """ Hierarchy should be formated as a tuple, for example ``("GO", "biological_process")``"""

        self.genes = set(genes)
        """ A set of genes. Genes are strings. """

        self.name = name
        """ Gene set name. """

        self.id = id
        """ Short gene set ID. """

        self.description = description
        """ Gene set description. """

        self.link = link
        """ Link to further information about this gene set. """

        self.organism = organism
        """ Organism as a NCBI taxonomy ID. """

        if pair:
            self.id, self.genes = pair[0], set(pair[1])

    """
    the following functions are needed for sets of gene sets to be able
    to assess equality
    """

    def __hash__(self):
        return self.id.__hash__() + self.name.__hash__()

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def size(self):
        return len(self.genes)

    def cname(self, source=True, name=True):
        """ Return a gene set name with hieararchy. """
        oname = self.id
        if source and self.hierarchy:
            oname = "[ " + ", ".join(self.hierarchy) + " ] " + oname
        if name and self.name:
            oname = oname + " " + self.name
        return oname

    def to_odict(self, source=True, name=True):
        """
        For backward compatibility. Return a gene set as a tuple
        (id, list of genes).
        """
        return self.cname(source=source, name=name), self.genes

    def __repr__(self):
        return "GeneSet(" + ", ".join( [ 
            "id=" + str(self.id),
            "genes=" + str(self.genes),
            "name=" + str(self.name),
            "link=" + str(self.link),
            "hierarchy=" + str(self.hierarchy)
        ]) + ")"

class GeneSetIDException(Exception):
    pass

class GeneSets(set):
    """ A collection of gene sets: contains :class:`GeneSet` objects. 
    """
    
    def __init__(self, input=None):
        """
        If `input` is a dictionary, the gene sets are converted to the current format.
        """
        if input != None and len(input) > 0:
            self.update(input)

    def update(self, input):
        if input.__class__.__name__ == "GeneSets": #HACK: because location can appear different
            super(GeneSets, self).update(input)
        else:
            prepared_genesets = [] #parse them all before adding,
                                   #so that it fails on error
            if hasattr(input, "items"):
                for i, g in input.items():
                    prepared_genesets.append(GeneSet(pair=(i, g)))
            else:
                for i in input:
                    if isinstance(i, GeneSet):
                        prepared_genesets.append(i)
                    else:
                        i, g = i
                        prepared_genesets.append(GeneSet(pair=(i, g)))

            for g in prepared_genesets:
                self.add(g)

    def to_odict(self):
        """ Return gene sets in old dictionary format. """
        return dict(gs.to_odict() for gs in self)

    def set_hierarchy(self, hierarchy):
        """ Sets hierarchy for all gene sets. """
        for gs in self:
            gs.hierarchy = hierarchy

    def __repr__(self):
        return "GeneSets(" + set.__repr__(self) + ")"

    def common_org(self):
        """ Return a common organism. """
        if len(self) == 0:
            raise GenesetRegException("Empty gene sets.")

        organisms = set(a.organism for a in self)

        try:
            return only_option(organisms)
        except:
            raise GenesetRegException("multiple organisms: " + str(organisms))

    def hierarchies(self):
        """ Return all hierarchies. """
        if len(self) == 0:
            raise GenesetRegException("Empty gene sets.")
        return set(a.hierarchy for a in self)

    def common_hierarchy(self):
        """ Return a common hierarchy. """
        hierarchies = self.hierarchies()

        def common_hierarchy1(hierarchies):
            def hier(l): return set(map(lambda x: x[:currentl], hierarchies))
            currentl = max(map(len, hierarchies))
            while len(hier(currentl)) > 1:
                currentl -= 1
            return only_option(hier(currentl))

        return common_hierarchy1(hierarchies)

    def split_by_hierarchy(self):
        """ Split gene sets by hierarchies. Return a list of :class:`GeneSets` objects. """
        hd = dict((h,GeneSets()) for h in  self.hierarchies())
        for gs in self:
            hd[gs.hierarchy].add(gs)
        return hd.values()


if __name__ == "__main__":
    rsf = orngServerFiles.ServerFiles(username=sys.argv[1], password=sys.argv[2])
    upload_genesets(rsf)
    pass






"""
Module for browsing any analyzing sets annotated with MeSH ontology.
"""

import Orange
from xml.sax import make_parser
from xml.dom import minidom
from xml.sax.handler import ContentHandler
from math import log, exp
from urllib import urlopen
from sgmllib import SGMLParser
import os.path

import orange
from Orange.orng import orngServerFiles

FUZZYMETAID = -12

import Orange.bio.utils.stats
HYPERG = Orange.bio.utils.stats.Hypergeometric()

class obiMeSH(object):

    def __init__(self):
        """
        Function will initialize obiMeSH module and (in case of no MeSH ontology data) download necessary data using orngServerFiles.
        """
        # print "SVN version"
        # we check if all files from MeSH directory are localy present. if not, download them
        self.reference = None
        self.cluster = None
        self.ratio = 1
        self.statistics = None
        self.calculated = False

        self.ref_att = "Unknown"
        self.clu_att = "Unknown"
        self.solo_att = "Unknown"

        # we calculate log(i!)
        self.dataLoaded = self.__loadOntologyFromDisk()

    def expandToFuzzyExamples(self, examples, att, a, b):
        """
        Function will return new 'fuzzy' example table. Every example from the input table will get two additional meta attributes ('fuzzy set' and 'u') \
        based on 'a' and 'b' threshold (lower and higher) and attribute 'att'. Attribute 'fuzzy set' indicates name of the fuzzy set while atribute 'u' \
        reflects example's degree of membership to particular fuzzy set. Note that input examples with values of 'att' lying on the (a,b) will be expanded \
        into two fuzzy examples.
        """
        mu = orange.FloatVariable("u")
        mv = orange.StringVariable("fuzzy set")
        examples.domain.addmeta(FUZZYMETAID, mu)
        examples.domain.addmeta(FUZZYMETAID - 1, mv)
        newexamples = []
        for j in range(0, len(examples)):
            i = examples[j]
            v = float(i[att])
            if v > a and v < b:  # we have to expand this example
                newexamples.append(i)
                i["fuzzy set"] = 'yes'
                i["u"] = (v - a) / (b - a)
                examples.append(i)
                examples[-1]["fuzzy set"] = "no"
                examples[-1]["u"] = (b - v) / (b - a)
            else:
                if v > a:  # u(yes) = 1.0
                    i["fuzzy set"] = 'yes'
                    i["u"] = 1.0
                else:  # u(no) = 1.0
                    i["fuzzy set"] = 'no'
                    i["u"] = 1.0
        return examples

    def findSubset(self, examples, meshTerms, callback=None, MeSHtype='term'):
        """
        Function findSubset will return new example table containing examples which are annotated with at least one MeSH term from list 'meshTerms'.
        """
        # clone
        newdata = orange.ExampleTable(examples.domain)
        self.solo_att = self.__findMeshAttribute(examples)
        ids = list()
        l = len(examples)
        c = 0.0
        meshTerms = list(set(meshTerms))
        # we couldn't find any mesh attribute
        if self.solo_att == "Unknown":
            return newdata
        if MeSHtype == 'term':
            for i in meshTerms:
                ids.extend(self.toID[i])
        else:
            ids = meshTerms
        for e in examples:
            try:
                if callback and c % 10 == 0:
                    callback(int(c * 100.0 / l))
                c = c + 1.0
                ends = eval(e[self.solo_att].value)
            except SyntaxError:
                # print "Error in parsing ", e[self.solo_att].value
                continue
            endids = list()
            for i in ends:
                if self.toID.has_key(i):
                    endids.extend(self.toID[i])
            #allnodes = self.__findParents(endids)
            allnodes = endids
            # calculate intersection
            isOk = False
            for i in allnodes:
                if ids.count(i) > 0:
                    isOk = True
                    break

            if isOk:	  # intersection between example mesh terms and observed term group is None
                newdata.append(e)
        return newdata

    def findTerms(self, ids, idType="cid", callback=None):
        """
        Function findTerms returns a dictionary containing annotated items from the list 'ids'.
        """
        ret = dict()
        if(not self.dataLoaded):
            print "Annotation and ontology has never been loaded!"
            return ret

        if idType == "cid":
            for i in ids:
                if self.fromCID.has_key(int(i)):  # maybe it is on our database
                    ret[int(i)] = self.fromCID[int(i)]
                else:   # no it is not, let's go on the internet - use pubChemAPI
                    r = pubChemAPI()
                    l = r.getMeSHterms(i)
                    ca = r.getPharmActionList(i)
                    l.extend(ca)  # we extend it with Pharmacological actions
                    #l = self.findTermsForCID(i)
                    self.fromCID[int(i)] = l
                    ret[int(i)] = l

                    # if len(l)>0:
                    # if we found something lets save it to a file
                    #   TODO: Never use __file__ to access data, but pkg_resources so that it works also in eggs
                    #	__dataPath = os.path.join(os.path.dirname(__file__), self.path)
                    #	fileHandle = open(os.path.join(__dataPath,'cid-annotation.dat'), 'a')
                    #	for s in l:
                    #		fileHandle.write('\n' + str(i) + ';' + s )
                    #	fileHandle.close()
            return ret
        elif idType == "pmid":  # FIXME PMID annotation
            database = self.fromPMID
        else:
            return ret

        for i in ids:
            if(database.has_key(int(i))):
                ret[i] = database[int(i)]
        return ret

    def findCIDSubset(self, examples, meshTerms):
        """
        Function findCIDSubset will return new list of examples (ids) containing examples which are annotated with at least one MeSH term from \
        list 'meshTerms'.
        """
        newdata = []
        self.solo_att = self.__findMeshAttribute(examples)
        ids = list()
        meshTerms = list(set(meshTerms))
        # we couldn't find any mesh attribute
        if self.solo_att == "Unknown":
            return newdata
        for e in examples:
            try:
                ends = eval(e[self.solo_att].value)
            except SyntaxError:
                continue
            for i in meshTerms:
                if i in ends:
                    newdata.append(e['cid'].value)
                    break
        return newdata

    def findFrequentTerms(self, data, minSizeInTerm, treeData=False, callback=None):
        """
        Function findFrequentTerms iterates thru examples in data. For every example it finds appropriate annotation (MeSH term). Result of this \
        function is a dictionary where keys corespond to MeSH terms ids and a value is a number of examples annotated with particular MeSH term.
        """
        # we build a dictionary 		meshID -> [description, noReference, [cids] ]
        self.statistics = dict()
        self.calculated = False
        self.solo_att = self.__findMeshAttribute(data)
        # post processing variables
        ret = dict()
        ids = []
        succesors = dict()		# for each term id -> list of succesors
        succesors["tops"] = []
        # if we can't identify mesh attribute we return empty data structures
        if self.solo_att == "Unknown":
            if treeData:
                return succesors, ret
            else:
                return ret
        # plain frequency
        t = 0.0
        n = len(data)
        for i in data:
            t = t + 1
            if callback and t % 10 == 0:
                callback(int(t * 100 / n))
            try:
                endNodes = list(set(eval(i[self.solo_att].value)))  # for every CID we look up for end nodes in mesh. for every end node we have to find its ID
            except SyntaxError:
                # print "Error in parsing ",i[self.solo_att].value
                continue
            # we find ID of end nodes
            endIDs = []
            for k in endNodes:
                if(self.toID.has_key(k)):  # this should be always true, but anyway ...
                    endIDs.extend(self.toID[k])
                else:
                    print "Current ontology does not contain MeSH term ", k, "."
            # we find id of all parents
            #allIDs = self.__findParents(endIDs)
            allIDs = endIDs
            for k in allIDs:  # for every meshID we update statistics dictionary
                if(not self.statistics.has_key(k)):  # first time meshID
                    self.statistics[k] = 0
                self.statistics[k] += 1  # counter increased
        # post processing
        for i in self.statistics.iterkeys():
            if(self.statistics[i] >= minSizeInTerm):
                ret[i] = self.statistics[i]
                ids.append(i)
        # we can also return tree data
        if treeData:  # we return also compulsory data for tree printing
            return self.__treeData(ids), ret
        else:
            return ret

    def __treeData(self, ids):
        succesors = dict()
        succesors["tops"] = []
        # we build a list of succesors. for each node we know which are its succesors in mesh ontology
        for i in ids:
            succesors[i] = []
            for j in ids:
                if(i != j and self.__isPrecedesor(i, j)):
                    succesors[i].append(j)
        # for each node from list above we remove its indirect succesors
        # only  i -1-> j   remain
        for i in succesors.iterkeys():
            succs = succesors[i]
            second_level_succs = []
            for k in succs:
                second_level_succs.extend(succesors[k])
            for m in second_level_succs:
                if succesors[i].count(m) > 0:
                    succesors[i].remove(m)
        # we make a list of top nodes
        tops = list(ids)
        for i in ids:
            for j in succesors[i]:
                tops.remove(j)
        # we pack tops table and succesors hash
        succesors["tops"] = tops
        return succesors

    def findEnrichedTerms(self, reference, cluster, pThreshold=0.05, treeData=False, callback=None, fuzzy=False):
        """
        Function findEnrichedTerms computes MeSH term enrichment based on a 'reference' and 'cluster' sets. It returns a dictionary where \
        keys are enriched (their p-value lower that 'pThreshold') MeSH terms. Key values are lists made of several items (MeSH term id, \
        MeSH term description, number of examples from the reference set, number of examples from the cluster set, p value, fold enrichment).
        """
        self.clu_att = self.__findMeshAttribute(cluster)
        self.ref_att = self.__findMeshAttribute(reference)
        if((not self.calculated or self.reference != reference or self.cluster != cluster) and self.ref_att != "Unknown" and \
           self.clu_att != "Unknown"):  # Do have new data? Then we have to recalculate everything.
            self.reference = reference
            self.cluster = cluster
            self.__calculateAll(callback, fuzzy)
        # declarations
        ret = dict()
        ids = []
        succesors = dict()		# for each term id -> list of succesors
        succesors["tops"] = []
        # if some attributes were unknown
        if (self.clu_att == "Unknown" or self.ref_att == "Unknown"):
            if treeData:
                return succesors, ret
            else:
                return ret
        for i in self.statistics.iterkeys():
            if(self.statistics[i][2] <= pThreshold):  # or self.statistics[i][4] <= pThreshold ): #
                ret[i] = self.statistics[i]
                ids.append(i)
        if treeData:
            return self.__treeData(ids), ret
        else:
            return ret

    def printMeSH(self, data, selection=["term", "r", "c", "p"], func=None):
        """
        Function printMeSH can be used to print result (dictionary) from the findEnrichedTerms and findFrequentTerms functions.
        """
        # first we calculate additional info for printing MeSH ontology
        info = self.__treeData(data.keys())
        for i in info["tops"]:
            self.__pp(0, i, info, data, selection, funct=func)

    def __pp(self, offset, item, relations, data, selection, funct=None):
        mapping = {"term": 0, "desc": 1, "r": 2, "c": 3, "p": 4, "fold": 5, "func": 6}
        for i in range(0, offset):
            print " ",
        if type(data[item]) == list:
            pval = "%.4g" % data[item][2]
            fold = "%.4g" % data[item][3]
            print_data = [self.toName[item], self.toDesc[self.toName[item]], str(data[item][0]), str(data[item][1]), str(pval), str(fold)]
            for i in selection:
                if i != "term":
                    print i + "=" + print_data[mapping[i]],
                else:
                    print print_data[mapping[i]],
            if funct != None:
                print " ", funct(print_data[0]),
            # print self.toName[item], " r=" + str(data[item][1])  +" c="+ str(data[item][2])  ," p=" + str(pval) + " fold=" + str(fold)
            print ""
        else:
            print self.toName[item], " freq=" + str(data[item])
        for i in relations[item]:
            self.__pp(offset + 2, i, relations, data, selection, funct=funct)

    def printHtmlMeSH(self, data, selection=["term", "r", "c", "p"], func=None):
        """
        Function printHtmlMeSH if used to print results (dictinary) from the findFrequentTerms and findFrequentTerms functins in HTML format. \
        Together with the MeSH ontology it prints data like number of examples, p-values (enrichment).
        """
        # first we calculate additional info for printing MeSH ontology
        info = self.__treeData(data.keys())
        w = {"term": "'95px'", "r": "'70px'", "c": "'70px'", "p": "'95px'"}
        print "<table>\n<tr>"
        for i in selection:
            print "<th width=" + w[i] + " align='left'>" + i + "</th>"
        if func != None:
            func("header", "")
        print "</tr>\n"
        for i in info["tops"]:
            self.__htmlpp(0, i, info, data, selection, funct=func)
        print "</table>"

    def __htmlpp(self, offset, item, relations, data, selection, funct=None):
        mapping = {"term": 0, "desc": 1, "r": 2, "c": 3, "p": 4, "fold": 5, "func": 6}
        print "<tr>"
        if type(data[item]) == list:
            pval = "%.4g" % data[item][2]
            fold = "%.4g" % data[item][3]
            print_data = [self.toName[item], self.toDesc[self.toName[item]], str(data[item][0]), str(data[item][1]), str(pval), str(fold)]
            for i in selection:
                print "<td>"
                if i == "term":
                    for l in range(0, offset):
                        print "&nbsp;",
                elif i == "p":
                    print '%(#)2.3e' % {'#': float(print_data[mapping[i]])} + "</td>",
                    continue
                print print_data[mapping[i]] + " &nbsp;</td>",

            if funct != None:
                print funct(print_data[0], item),

            # print self.toName[item], " r=" + str(data[item][1])  +" c="+ str(data[item][2])  ," p=" + str(pval) + " fold=" + str(fold)
            print "</tr>"
        else:
            print self.toName[item], " freq=" + str(data[item])

        for i in relations[item]:
            self.__htmlpp(offset + 2, i, relations, data, selection, funct=funct)

    def parsePubMed(self, filename, attributes=["pmid", "title", "abstract", "mesh"], skipExamplesWithout=["mesh"]):
        """
        Function parsePubMed can be used to parse (into Orange example table) PubMed search results (in XML).
        """
        parser = make_parser()
        handler = pubMedHandler()
        parser.setContentHandler(handler)
        parser.parse(open(filename))
        atts = []
        for i in attributes:
            atts.append(orange.StringVariable(i))
        domain = orange.Domain(atts, 0)
        data = orange.ExampleTable(domain)
        print data.domain.attributes
        mapping = {"pmid": 0, "title": 1, "abstract": 2, "mesh": 3, "affilation": 4}
        for i in handler.articles:
            r = []
            skip = False
            for f in attributes:
                if skipExamplesWithout.count(f) > 0:
                    if (f == "mesh" and len(i[mapping[f]]) == 0) or str(i[mapping[f]]) == "":
                        skip = True
                r.append(str(i[mapping[f]]))
            if not skip:
                data.append(r)
        return data

    def __findParents(self, endNodes):
        """for each end node in endNodes function finds all nodes on the way up to the root"""
        res = []
        for n in endNodes:
            tmp = n
            res.append(tmp)
            for i in range(n.count(".")):
                tmp = tmp.rstrip("1234567890").rstrip(".")
                if(tmp not in res):
                    res.append(tmp)
        """for i in res:
			nn = self.toID[self.toName[i]]
			for t in nn:
				if not t in res:
					res.append(t) """
        return res

    def __findMeshAttribute(self, data):
        """
        Function tries to find attribute which contains list of MeSH terms.
        """
        # we get a list of attributes
        dom = data.domain.attributes
        for i in dom:			  # for each attribute
            if i.varType == 6:
                for k in data:		   # for each domain
                    att = str(i.name)
                    try:										 # we try to use eval()
                        r = eval(str(k[att].value))
                        if type(r) == list:		 # attribute type should be list
                            if self.dataLoaded:		 # if ontology is loaded we perform additional test
                                for i in r:
                                    if self.toID.has_key(i): return att
                            else:				   # otherwise we return list attribute
                                return att
                    except SyntaxError:
                        continue
                    except NameError:
                        continue
        print "Program was unable to determinate MeSH attribute."
        return "Unknown"

    def __isPrecedesor(self, a, b):
        """ function returns true if in Mesh ontology exists path from term id a to term id b """
        if b.count(a) > 0:
            return True
        return False

    def __calculateAll(self, callback, fuzzy):
        """calculates all statistics"""
        # we build a dictionary 		meshID -> [description, noReference,noCluster, enrichment, deprivement, [cids] ]
        # print "ok"
        self.statistics = dict()
        if fuzzy:
            n = 0
            for i in self.reference:
                n = n + float(i['u'])
            cln = 0
            for i in self.cluster:
                cln = cln + float(i['u'])

        else:
            n = len(self.reference) 										# reference size
            cln = len(self.cluster)											# cluster size
        # frequency from reference list
        r = 0.0
        for i in self.reference:
            if callback and r % 10 == 0:
                r += 1.0
                callback(int(100 * r / (n + cln)))
            try:
                endNodes = list(set(eval(i[self.ref_att].value)))  # for every CID we look up for end nodes in mesh. for every end node we have to find its ID
            except SyntaxError:					 # where was a parse error
                print "Error in parsing ", i[self.ref_att].value
                if fuzzy:
                    n = n - float(i["u"])
                else:
                    n = n - 1
                continue
            # we find ID of end nodes
            endIDs = []
            for k in endNodes:
                if(self.toID.has_key(k)):					# this should be always true, but anyway ...
                    endIDs.extend(self.toID[k])
                else:
                    print "Current ontology does not contain MeSH term ", k, "."
            # endIDs may be empty > in this case we can skip this example
            if len(endIDs) == 0:
                if fuzzy:
                    n = n - float(i["u"])
                else:
                    n = n - 1
                continue
            # we find id of all parents
            # allIDs = self.__findParents(endIDs) PATCH
            allIDs = endIDs
            for k in list(set(allIDs)):				# for every meshID we update statistics dictionary
                if(not self.statistics.has_key(k)):			# first time meshID
                    self.statistics[k] = [0, 0, 0.0, 0.0]
                if fuzzy:
                    self.statistics[k][0] += float(i["u"])
                else:
                    self.statistics[k][0] += 1  # increased noReference
        # frequency from cluster list
        r = 0.0
        for i in self.cluster:
            try:
                if callback and r % 10 == 0:
                    r += 1.0
                    callback(int(100 * r / (n + cln)))
                endNodes = list(set(eval(i[self.clu_att].value)))  # for every CID we look up for end nodes in mesh. for every end node we have to find its ID
            except SyntaxError:
                # print "Error in parsing ",i[self.clu_att].value
                if fuzzy:
                    cln = cln - float(i["u"])
                else:
                    cln = cln - 1
                continue
            # we find ID of end nodes
            endIDs = []
            for k in endNodes:
                if(self.toID.has_key(k)):
                    endIDs.extend(self.toID[k])  # for every endNode we add all corensponding meshIDs
            # endIDs may be empty > in this case we can skip this example
            if len(endIDs) == 0:
                if fuzzy:
                    cln = cln - float(i["u"])
                else:
                    cln = cln - 1
                continue
            # we find id of all parents
            #allIDs = self.__findParents(endIDs)
            allIDs = endIDs
            for k in list(set(allIDs)):  # for every meshID we update statistics dictionary
                if self.statistics.has_key(k):
                    if fuzzy:
                        self.statistics[k][1] += float(i["u"])
                    else:
                        self.statistics[k][1] += 1  # increased noCluster
        self.ratio = float(cln) / float(n)
        # enrichment
        for i in self.statistics.iterkeys():
            self.statistics[i][2] = HYPERG.p_value(int(self.statistics[i][1]), int(n), int(cln), int(self.statistics[i][0]))
            self.statistics[i][3] = float(self.statistics[i][1]) / float(self.statistics[i][0]) / self.ratio   # fold enrichment
        self.calculated = True

    def __loadOntologyFromDisk(self):
        """
        Function loads MeSH ontology and chemical annotation into internal data structures.
        """
        self.toID = dict()  # name -> [IDs] Be careful !!! One name can match many IDs!
        self.toName = dict()  # ID -> name
        self.toDesc = dict()  # name -> description
        self.fromCID = dict()  # cid -> term id
        self.fromPMID = dict()  # pmid -> term id

        d = file(orngServerFiles.localpath_download('MeSH', 'mesh-ontology.dat'))
        f = file(orngServerFiles.localpath_download('MeSH', 'cid-annotation.dat'))

        # loading ontology graph
        t = 0
        for i in d:
            t += 1
            parts = i.split("\t")  # delimiters are tabs
            if(len(parts) != 3):
                print "error reading ontology ", parts[0]
            parts[2] = parts[2].rstrip("\n\r")
            ids = parts[1].split(";")
            self.toID[parts[0]] = ids  # append additional ID
            self.toDesc[parts[0]] = parts[2]
            for r in ids:
                self.toName[r] = parts[0]
            # loading cid -> mesh
        for i in f:
            parts = i.split(";")		# delimiters are tabs
            if(len(parts) != 2):
                print "error reading ontology ", parts[0]
            parts[1] = parts[1].rstrip("\n\r")
            cid = int(parts[0])
            if self.fromCID.has_key(cid):
                self.fromCID[cid].append(parts[1])
            else:
                self.fromCID[cid] = [parts[1]]
        # loading pmid -> mesh, TODO
        # print "Current MeSH ontology contains ", t, " mesh terms."
        return True


class pubMedHandler(ContentHandler):

    def __init__(self):
        self.state = 0  # 0 start state, 1 pmid, 2 title, 3 abstract, 4 mesh
        self.articles = []
        self.pmid = "0"
        self.title = ""
        self.mesh = list()
        self.abstract = ""
        self.affiliation = ""

    def startElement(self, name, attributes):
        # print "parsam ", name
        if name == "PubmedArticle":
            self.pmid = ""
            self.abstract = ""
            self.title = ""
            self.mesh = []
        if name == "PMID":
            self.state = 1
        if name == "ArticleTitle":
            self.state = 2
        if name == "AbstractText":
            self.state = 3
        if name == "DescriptorName":
            self.state = 4
            self.mesh.append("")
        if name == "Affiliation":
            self.state = 5

    def characters(self, data):
        if self.state == 1:
            self.pmid += data
        if self.state == 2:
            self.title += data.encode("utf-8")
        if self.state == 3:
            self.abstract += data.encode("utf-8")
        if self.state == 4:
            self.mesh[-1] += data.encode("utf-8")
        if self.state == 5:
            self.affiliation += data.encode("utf-8")

    def endElement(self, name):
        # print "   koncujem ", name
        self.state = 0
        if name == "PubmedArticle":
            self.articles.append([self.pmid, self.title, self.abstract, self.mesh, self.affiliation])


class MappedMeSHParser(SGMLParser):

    def reset(self):
        self.pieces = []
        self.terms = []
        self.foundMeSH = False
        self.nextIsTerm = False
        self.endTags = ['Display', 'Write to the Help Desk', 'Previous Indexing:', 'Entry Terms:', 'Pharmacologic Action:']
        SGMLParser.reset(self)

    def unknown_starttag(self, tag, attrs):
        strattrs = "".join([' %s="%s"' % (key, value) for key, value in attrs])
        if self.foundMeSH and tag == 'a':
            self.nextIsTerm = True

    def handle_data(self, text):
        text = text.strip()
        if text == '':
            return
        if text == 'Heading Mapped to:':
            self.foundMeSH = True
        if self.endTags.count(text) > 0:
            self.foundMeSH = False
        elif self.nextIsTerm:
            self.terms.append(text)
            self.nextIsTerm = False


class PubChemMeSHParser(SGMLParser):

    def reset(self):
        self.next = 0
        self.nextLink = ''
        self.directTerms = []
        self.indirectTerms = []
        self.foundMeSH = False
        self.foundIndirectMeSH = 0
        SGMLParser.reset(self)
        # strategy as follows
        # Beetween strings "Drug and Chemical Info" and ("Pharmalogical Action" or "PubMed via MeSH" or "PubMed MeSH Keyword Summary") find hyperlinks. Based on title attribute we can distingue direct MeSH terms beetween mapped terms.

    def unknown_starttag(self, tag, attrs):
        # if self.foundMeSH:
        # print 'tag ', tag, attrs
        if tag == 'a' and len(attrs) > 2 and attrs[0][0] == 'name' and attrs[0][1] == 'gomesh':  # print attrs
            self.nextLink = attrs[1][1]

    def handle_data(self, text):
        text = text.strip()
        if text == '':
            return
        # print 'data ', text
        if self.foundMeSH:
            # print '*'
            self.directTerms.append(text)

        if self.foundIndirectMeSH == 2:
            # print "indirect term", text, self.nextLink
            self.indirectTerms.append((text, self.nextLink))
            self.foundIndirectMeSH = 0

        if self.foundIndirectMeSH == 1:
            self.foundIndirectMeSH = 2

        if text == 'Drug and Chemical Information:':
            self.foundIndirectMeSH = 1

        if text == 'Pharmacological Action' or text == 'Medication Information':
            self.foundIndirectMeSH = 0
        if text == "Chemical Classification":
            self.foundMeSH = True
        elif (text == "Safety and Toxicology" or text == "Literature" or text == "Classification" or text == "PubMed via MeSH" or text == "PubMed MeSH Keyword Summary"):
            self.foundMeSH = False
            if len(self.directTerms) > 0:
                self.directTerms.pop()


class SmilesParser(SGMLParser):

    def reset(self):
        self.nextSmile = False
        self.smiles = ""
        SGMLParser.reset(self)

    def unknown_starttag(self, tag, attrs):
        # print tag, " ", attrs
        if tag == "item":
            for (i, j) in attrs:
                if i == "name" and j == "CanonicalSmile":
                    self.nextSmile = True

    def handle_data(self, text):
        if self.nextSmile:
            self.smiles = text
            self.nextSmile = False


class CIDsParser(SGMLParser):

    def reset(self):
        self.cid = False
        self.cids = []
        SGMLParser.reset(self)

    def unknown_starttag(self, tag, attrs):
        # print tag, " ", attrs
        if tag == "id":
            self.cid = True

    def unknown_endtag(self, tag):
        # print tag,
        if tag == "id":
            self.cid = False

    def handle_data(self, text):
        # print text
        if self.cid and not text == '\n\t\t':
            # print int(text.strip())
            self.cids.append(int(text))


class FormulaParser(SGMLParser):

    def reset(self):
        self.nextFormula = False
        self.formula = ""
        SGMLParser.reset(self)

    def unknown_starttag(self, tag, attrs):
        # print tag, " ", attrs
        if tag == "item":
            for (i, j) in attrs:
                if i == "name" and j == "MolecularFormula":
                    self.nextFormula = True

    def handle_data(self, text):
        if self.nextFormula:
            self.formula = text
            self.nextFormula = False


class CIDSParser(SGMLParser):

    def reset(self):
        self.nextCID = False
        self.cids = []
        SGMLParser.reset(self)

    def unknown_starttag(self, tag, attrs):
        # print tag, " ", attrs
        if tag == "item":
            for (i, j) in attrs:
                if i == "name" and j == "int":
                    self.nextCID = True

    def handle_data(self, text):
        if self.nextCID:
            self.cids.append(text)
            self.nextCID = False


class pubChemAPI(object):

    """
    Class pubChemAPI is used to simplift the access to the pubChem database.
    """

    def __init__(self):
        self.data = ""
        self.identifier = ""
        self.database = ""

    def getSMILE(self, id, typ):
        """
        Function getSMILE takes chemicals id and its type and returns a SMILES representation.
        """
        dbs = {"sid": "pcsubstance", "cid": "pccompound"}
        if not dbs.has_key(typ):
            return "Unknown identifier"
        # maybe we already have the data ...
        if id == self.identifier and dbs[typ] == self.database:
            raw = self.data
        else:
            url2fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?db=" + dbs[typ] + "&id=" + str(id)
            # print url2fetch
            d = urlopen(url2fetch)
            raw = d.read()
            self.data = raw
            self.identifier = id
            self.database = dbs[typ]
        pr = SmilesParser()
        pr.feed(raw)
        data = pr.smiles
        pr.close()
        return data

    def getMeSHterms(self, cid):
        """
        Functions getMeSHterms tries to find MeSH annotation for given the CID in the pubChem database.
        """
        usock = urlopen("http://pubchem.ncbi.nlm.nih.gov/summary/summary.cgi?cid=" + str(cid) + "&viewopt=PubChem&hcount=100&tree=t#MeSH")
        parser = PubChemMeSHParser()
        text = usock.read()
        # print text
        parser.feed(text)
        usock.close()
        parser.close()
        allTerms = parser.directTerms
        # print allTerms
        for (k, i) in parser.indirectTerms:
            # print k, i
            # continue
            usock = urlopen(i)
            parser = MappedMeSHParser()
            parser.feed(usock.read())
            allTerms.extend(parser.terms)
            usock.close()
            parser.close()
        # from allTerms make a set
        allTerms = list(set(allTerms))
        return allTerms

    def getMolecularFormula(self, id, typ):
        """
        Functions getMolecularFormula tries to find molecular formula for the given the chemical id and type in the pubChem database.
        """
        dbs = {"sid": "pcsubstance", "cid": "pccompound"}
        if not dbs.has_key(typ):
            return "Unknown identifier"
        # maybe we already have the data ...
        if id == self.identifier and dbs[typ] == self.database:
            raw = self.data
        else:
            url2fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?db=" + dbs[typ] + "&id=" + str(id)
            # print url2fetch
            d = urlopen(url2fetch)
            raw = d.read()
            self.data = raw
            self.identifier = id
            self.database = dbs[typ]
        pr = FormulaParser()
        pr.feed(raw)
        data = pr.formula
        pr.close()
        return data

    def getCIDfromSID(self, sid):
        """
        Functions getCIDfromSID converts compound id to substance id.
        """
        dbs = {"sid": "pcsubstance", "cid": "pccompound"}
        if not dbs.has_key("sid"):
            return "Unknown identifier"
        # maybe we already have the data ...
        if sid == self.identifier and dbs["sid"] == self.database:
            raw = self.data
        else:
            url2fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?db=" + dbs["sid"] + "&id=" + str(sid)
            # print url2fetch
            d = urlopen(url2fetch)
            raw = d.read()
            self.data = raw
            self.identifier = sid
            self.database = dbs["sid"]
        pr = CIDSParser()
        pr.feed(raw)
        data = pr.cids
        pr.close()
        return data

    def getPharmActionList(self, cid):
        """
        Functions getPharmActionList finds pharmacological MeSH annotation for the given CID.
        <Item Name="PharmActionList" Type="List">
        """
        dbs = {"sid": "pcsubstance", "cid": "pccompound"}
        # maybe we already have the data ...
        if cid == self.identifier and dbs["cid"] == self.database:
            raw = self.data
        else:
            url2fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?db=" + dbs["cid"] + "&id=" + str(cid)
            d = urlopen(url2fetch)
            raw = d.read()
        # print url2fetch
        self.data = raw
        self.identifier = cid
        self.database = dbs["cid"]
        ndata = raw.replace('\t', '').replace('\n', '')
        xmldoc = minidom.parseString(ndata)
        items = xmldoc.getElementsByTagName('Item')
        data = []
        for i in items:
            if i.attributes['Name'].value == 'PharmActionList':
                terms = i.childNodes
                for t in terms:
                    data.append(str(t.childNodes[0].data))
        return data

    def getCIDs(self, name, weight):
        """
        Functions getCIDs tries to find correct chemical id based on given chemical name and weight.
        """
        offset = 0.005
        url2fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pccompound&term="
        url2fetch += name.replace(" ", "%20") + "%20" + str(weight * (1 - offset)) + ":" + str(weight * (1 + offset)) + "[mw]&retmax=10000"
        # print url2fetch
        d = urlopen(url2fetch)
        raw = d.read()
        e = CIDsParser()
        e.feed(raw)
        return e.cids

    def getCIDs(self, name):
        """
        Functions getCIDs tries to find correct chemical id based on given chemical name.
        """
        # print "k"
        url2fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pccompound&term="
        url2fetch += name.replace(" ", "%20") + "&retmax=10000"
        # print url2fetch
        d = urlopen(url2fetch)
        raw = d.read()
        e = CIDsParser()
        e.feed(raw)
        return e.cids

    def __strip_ml_tags__(self, in_text):
        s_list = list(in_text)
        i, j = 0, 0
        while i < len(s_list):
            if s_list[i] == '<':
                while s_list[i] != '>':
                    s_list.pop(i)
                s_list.pop(i)
            else:
                i = i + 1
        join_char = ''
        return join_char.join(s_list)



"""
==============================================
KEGG - Kyoto Encyclopedia of Genes and Genomes
==============================================

:mod:`kegg` is a python module for accessing `KEGG (Kyoto Encyclopedia
of Genes and Genomes) <http://www.genome.jp/kegg/>`_ using its web services.

.. note:: This module requires `slumber`_ and `requests`_ packages.

.. _`slumber`: https://pypi.python.org/pypi/slumber/

.. _`requests`: https://pypi.python.org/pypi/requests


>>> from Orange.bio.kegg import *
>>> # Create a KEGG Genes database interface
>>> genome = KEGGGenome()
>>> # List all available entry ids
>>> keys = genome.keys()
>>> print keys[0]
T01001
>>> # Retrieve the entry for the key.
>>> entry = genome[keys[0]]
>>> print entry.entry_key
T01001
>>> print entry.definition
Homo sapiens (human)
>>> print str(entry)
ENTRY       T01001            Complete  Genome
NAME        hsa, HUMAN, 9606
DEFINITION  Homo sapiens (human)
...

The :class:`Organism` class can be a convenient starting point
for organism specific databases.

>>> organism = Organism("Homo sapiens")  # searches for the organism by name
>>> print organism.org_code  # prints the KEGG organism code
hsa
>>> genes = organism.genes  # get the genes database for the organism
>>> gene_ids = genes.keys() # KEGG gene identifiers
>>> entry = genes["hsa:672"]
>>> print entry.definition
breast cancer 1, early onset
>>> print entry  # print the entry in DBGET database format.
ENTRY       672               CDS       T01001
NAME        BRCA1, BRCAI, BRCC1, BROVCA1, IRIS, PNCA4, PPP1R53, PSCP, RNF53
DEFINITION  breast cancer 1, early onset
...

"""
from __future__ import absolute_import

import os
import sys
import urllib2
import threading

from collections import defaultdict
from itertools import chain
from datetime import datetime
from contextlib import contextmanager

from Orange.utils import lru_cache
from Orange.utils import progress_bar_milestones
from Orange.utils import (
    deprecated_keywords, deprecated_attribute, deprecated_function_name
)

from .. import utils, taxonomy as obiTaxonomy
obiProb = utils.stats

from . import databases
from . import entry

from .brite import BriteEntry, Brite

from . import api
from . import conf
from . import pathway

KEGGGenome = databases.Genome
KEGGGenes = databases.Genes
KEGGEnzyme = databases.Enzyme
KEGGReaction = databases.Reaction
KEGGPathways = databases.Pathway
KEGGCompound = databases.Compound

KEGGBrite = Brite
KEGGBriteEntry = BriteEntry

KEGGPathway = pathway.Pathway

DEFAULT_CACHE_DIR = conf.params["cache.path"]


class Organism(object):
    """
    A convenience class for retrieving information regarding an
    organism in the KEGG Genes database.

    :param str org: KEGG organism code (e.g. "hsa", "sce"). Can also be a
        descriptive name (e.g. 'yeast', "homo sapiens") in which case the
        organism code will be searched for by using KEGG `find` api.

    .. seealso::

        :func:`organism_name_search`
            Search KEGG for an organism code

    """
    def __init__(self, org, genematcher=None):
        self.org_code = self.organism_name_search(org)
        self.genematcher = genematcher
        self.api = api.CachedKeggApi()

    @property
    def org(self):
        """
        KEGG organism code.
        """
        return self.org_code

    @property
    def genes(self):
        """
        An :class:`~.databases.Genes` database instance for this organism.
        """
        # TODO: This should not be a property but a method.
        # I think it was only put here as back compatibility with old obiKEGG.
        if not hasattr(self, "_genes"):
            genes = KEGGGenes(self.org_code)
            self._genes = genes
        return self._genes

    def gene_aliases(self):
        """
        Return a list of sets of equal genes (synonyms) in KEGG for
        this organism.

        .. note::

            This only includes 'ncbi-geneid' and 'ncbi-gi' records
            from the KEGG Genes DBLINKS entries.

        """
        definitions = self.api.list(self.org_code)
        ncbi_geneid = self.api.conv(self.org_code, "ncbi-geneid")
        ncbi_gi = self.api.conv(self.org_code, "ncbi-gi")

        aliases = defaultdict(set)

        for entry_id, definition in definitions:
            # genes entry id without the organism code
            aliases[entry_id].add(entry_id.split(":", 1)[1])
            # all names in the NAME field (KEGG API list returns
            # 'NAME; DEFINITION') fields for genes
            names = definition.split(";")[0].split(",")
            aliases[entry_id].update([name.strip() for name in names])

        for source_id, target_id in chain(ncbi_geneid, ncbi_gi):
            aliases[target_id].add(source_id.split(":", 1)[1])

        return [set([entry_id]).union(names)
                for entry_id, names in aliases.iteritems()]

    def pathways(self, with_ids=None):
        """
        Return a list of all pathways for this organism.
        """
        if with_ids is not None:
            return self.api.get_pathways_by_genes(with_ids)
        else:
            return [p.entry_id for p in self.api.list_pathways(self.org_code)]

    def list_pathways(self):
        """
        List all pathways for this organism.

        .. deprecated: 2.5
            Use :func:`pathways` instead.

        """
        # NOTE: remove/deprecate and use pathways()
        return self.pathways()

    def get_linked_pathways(self, pathway_id):
        self.api.get_linked_pathways(pathway_id)

    def enzymes(self, genes=None):
        raise NotImplementedError()

    def get_enriched_pathways(self, genes, reference=None,
                              prob=obiProb.Binomial(), callback=None):
        """
        Return a dictionary with enriched pathways ids as keys
        and (list_of_genes, p_value, num_of_reference_genes) tuples
        as items.

        """
        if reference is None:
            reference = self.genes.keys()
        reference = set(reference)

        allPathways = defaultdict(lambda: [[], 1.0, []])
        milestones = progress_bar_milestones(len(genes), 100)
        pathways_db = KEGGPathways()

        pathways_for_gene = []
        for i, gene in enumerate(genes):
            pathways_for_gene.append(self.pathways([gene]))
            if callback and i in milestones:
                callback(i * 50.0 / len(genes))

        # pre-cache for speed
        pathways_db.pre_cache([pid for pfg in pathways_for_gene
                               for pid in pfg])
        for i, (gene, pathways) in enumerate(zip(genes, pathways_for_gene)):
            for pathway in pathways:
                if pathways_db.get_entry(pathway).gene:
                    allPathways[pathway][0].append(gene)
            if callback and i in milestones:
                callback(50.0 + i * 50.0 / len(genes))

        pItems = allPathways.items()

        for i, (p_id, entry) in enumerate(pItems):
            pathway = pathways_db.get_entry(p_id)
            entry[2].extend(reference.intersection(pathway.gene or []))
            entry[1] = prob.p_value(len(entry[0]), len(reference),
                                    len(entry[2]), len(genes))
        return dict([(pid, (genes, p, len(ref)))
                     for pid, (genes, p, ref) in allPathways.items()])

    def get_genes_by_enzyme(self, enzyme):
        enzyme = KEGGEnzyme().get_entry(enzyme)
        return enzyme.genes.get(self.org_code, []) if enzyme.genes else []

    def get_genes_by_pathway(self, pathway_id):
        return KEGGPathway(pathway_id).genes()

    def get_enzymes_by_pathway(self, pathway_id):
        return KEGGPathway(pathway_id).enzymes()

    def get_compounds_by_pathway(self, pathway_id):
        return KEGGPathway(pathway_id).compounds()

    def get_pathways_by_genes(self, gene_ids):
        """ Pathways that include all genes in gene_ids. """
        l = self.api.get_genes_pathway_organism(self.org_code)
        gene_ids = set(gene_ids)
        gtp = defaultdict(set)
        for a,b in l:
            gtp[a].add(b)
        pathways = [ gtp[g] for g in gene_ids ]
        pathways = reduce(set.intersection, pathways)
        return sorted(pathways)

    def get_pathways_by_enzymes(self, enzyme_ids):
        enzyme_ids = set(enzyme_ids)
        pathways = [KEGGEnzyme()[id].pathway for id in enzyme_ids]
        pathways = reduce(set.union, pathways, set())
        return [id for id in pathways
                if enzyme_ids.issubset(KEGGPathway(id).enzymes())]

    def get_pathways_by_compounds(self, compound_ids):
        compound_ids = set(compound_ids)
        pathways = [KEGGCompound()[id].pathway for id in compound_ids]
        pathways = reduce(set.union, pathways, set())
        return [id for id in pathways
                if compound_ids.issubset(KEGGPathway(id).compounds())]

    def get_enzymes_by_compound(self, compound_id):
        return KEGGCompound()[compound_id].enzyme

    def get_enzymes_by_gene(self, gene_id):
        return self.genes[gene_id].enzymes

    def get_compounds_by_enzyme(self, enzyme_id):
        return self._enzymes_to_compounds.get(enzyme_id)

    @deprecated_keywords({"caseSensitive": "case_sensitive"})
    def get_unique_gene_ids(self, genes, case_sensitive=True):
        """
        Return a tuple with three elements. The first is a dictionary
        mapping from unique geneids to gene names in genes, the second
        is a list of conflicting gene names and the third is a list of
        unknown genes.

        """
        unique, conflicting, unknown = {}, [], []
        for gene in genes:
            names = self.genematcher.match(gene)
            if len(names) == 1:
                unique[names[0]] = gene
            elif len(names) == 0:
                unknown.append(gene)
            else:
                conflicting.append(gene)
        return unique, conflicting, unknown

    @deprecated_function_name
    def get_genes(self):
        return self.genes

    @classmethod
    def organism_name_search(cls, name):
        return organism_name_search(name)

    @classmethod
    def organism_version(cls, name):
        name = cls.organism_name_search(name)
        with _global_genome_instance() as genome:
            info = genome.api.info(name)
            return info.release

    def _set_genematcher(self, genematcher):
        setattr(self, "_genematcher", genematcher)

    def _get_genematcher(self):
        if getattr(self, "_genematcher", None) is None:
            from .. import obiGene
            if self.org_code == "ddi":
                self._genematcher = obiGene.matcher(
                    [obiGene.GMKEGG(self.org_code), obiGene.GMDicty(),
                     [obiGene.GMKEGG(self.org_code), obiGene.GMDicty()]]
                )
            else:
                self._genematcher = obiGene.matcher(
                    [obiGene.GMKEGG(self.org_code)])

            self._genematcher.set_targets(self.genes.keys())
        return self._genematcher

    genematcher = property(_get_genematcher, _set_genematcher)


KEGGOrganism = Organism


def organism_name_search(name):
    """
    Search for a organism by `name` and return it's KEGG organism code.
    """
    with _global_genome_instance() as genome:
        try:
            name = genome.org_code_to_entry_key(name)
        except ValueError:
            pass

        if name not in genome:
            ids = genome.search(name)
            if ids:
                name = ids.pop(0) if ids else name
            else:
                raise obiTaxonomy.UnknownSpeciesIdentifier(name)

        try:
            return genome[name].organism_code
        except KeyError:
            raise obiTaxonomy.UnknownSpeciesIdentifier(name)


def pathways(org):
    """
    Return a list of all KEGG pathways for an KEGG organism code `org`.
    """
    return KEGGPathway.list(org)


def from_taxid(taxid):
    """
    Return a KEGG organism code for a an NCBI Taxonomy id string `taxid`.
    """
    with _global_genome_instance() as genome:
        res = genome.search(taxid)
        for r in res:
            e = genome[r]

            if e.taxid in [taxid, genome.TAXID_MAP.get(taxid, taxid)]:
                return e.org_code()

        return None


def to_taxid(name):
    """
    Return a NCBI Taxonomy id for a given KEGG Organism name
    """
    with _global_genome_instance() as genome:
        if name in genome:
            return genome[name].taxid

        keys = genome.search(name)
        if keys:
            return genome[keys[0]].taxid
        else:
            return None


def _global_genome_instance():
    genome = getattr(_global_genome_instance, "_genome", None)
    if genome is None:
        genome = KEGGGenome()
        genome._lock = threading.RLock()
        _global_genome_instance._genome = genome

    @contextmanager
    def instance_locked(instance, lock):
        lock.acquire()
        try:
            yield instance
        finally:
            lock.release()

    return instance_locked(genome, genome._lock)


def main():
    KEGGGenome()
    import doctest
    extraglobs = {"api": KeggApi()}
    doctest.testmod(optionflags=doctest.ELLIPSIS, extraglobs=extraglobs)


if __name__ == "__main__":
    sys.exit(main())



import sys, os
import urllib2
import shutil
import re
import Orange

from collections import defaultdict

class disease(object):
    """ A disease in the OMIM database.
    """
    regex = re.compile(r'(?P<name>.*?),? (?P<id>[0-9]{3,6} )?(?P<m1>\([123?]\) )?(?P<m2>\([123?]\) )? *$')
    __slots__ = ["name", "id", "mapping"]
    def __init__(self, morbidmap_line):
        string = morbidmap_line.split("|", 1)[0]
        match = self.regex.match(string)
#        print string
#        print match.groups()
        self.name, self.id, self.mapping = [s.strip() if s else s for s in match.groups()[:3]]
        if match.group("m2"):
            self.mapping += " " + match.group("m2").strip()
                                                                                
class OMIM(object):
    VERSION = 1
    DEFAULT_DATABASE_PATH = Orange.utils.serverfiles.localpath("OMIM")
    def __init__(self, local_database_path=None):
        self.local_database_path = local_database_path if local_database_path is not None else self.DEFAULT_DATABASE_PATH
  
        if self.local_database_path == self.DEFAULT_DATABASE_PATH:
            filename = Orange.utils.serverfiles.localpath_download("OMIM", "morbidmap")
        else:
            filename = os.path.join(self.local_database_path, "morbidmap")

        self.load(filename)
    
    @classmethod
    def download_from_NCBI(cls, file=None):
        if isinstance(file, basestring):
            file = open(file, "wb")
        stream = urllib2.urlopen("ftp://ftp.ncbi.nih.gov/repository/OMIM/ARCHIVE/morbidmap")
        shutil.copyfileobj(stream, file, length=10)
        file.close()

    @classmethod
    def get_instance(cls):
        if not hasattr(cls, "_shared_dict"):
            omim = OMIM()
            cls._shared_dict = omim.__dict__
        instance = OMIM.__new__(OMIM)
        instance.__dict__ = cls._shared_dict
        return instance 
    
    def load(self, filename):
        file = open(filename, "rb")
        lines = file.read().splitlines()
        self._disease_dict = dict([(disease(line), line) for line in lines if line])
    
    def diseases(self):
        return self._disease_dict.keys()
    
    def genes(self):
        return sorted(set(reduce(list.__add__, [self.disease_genes(disease) for disease in self.diseases()], [])))
    
    def disease_genes(self, disease):
        return self._disease_dict[disease].split("|")[1].split(", ")
    
    def gene_diseases(self):
        d = defaultdict(set)
        for disease, genes in [(disease, self.disease_genes(disease)) for disease in self.diseases()]:
            for gene in genes:
                d[gene].add(disease)
        return d

def diseases():
    """ Return all disease descriptors.
    """
    return OMIM.get_instance().diseases()

def genes():
    """ Return a set of all genes referenced in OMIM.
    """
    return OMIM.get_instance().genes()

def disease_genes(disease):
    """ Return a set of all genes referenced by disease in OMIM.
    """
    return OMIM.get_instance().disease_genes(disease)

def gene_diseases():
    """ Return a dictionary {gene: set(disease_objects for gene), ...}.
    """
    return OMIM.get_instance().gene_diseases()

